@@ -181,7 +181,6 @@ create_hig_frame (CalendarWindow *calwin,
                   GCallback   callback)
 {
         GtkWidget *vbox;
-        GtkWidget *label;
         GtkWidget *hbox;
         char      *bold_title;
         GtkWidget *expander;
@@ -211,9 +210,10 @@ create_hig_frame (CalendarWindow *calwin,
 	g_signal_connect (hbox, "add", G_CALLBACK (add_child), expander);
 
         if (button_label) {
+                GtkWidget *label;
                 GtkWidget *button_box;
                 GtkWidget *button;
-                gchar *text;
+                gchar     *text;
 
                 button_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
                 gtk_widget_show (button_box);
@@ -264,7 +264,7 @@ calendar_window_pack_locations (CalendarWindow *calwin, GtkWidget *vbox)
 	gtk_widget_show (calwin->priv->locations_list);
 	gtk_container_add (GTK_CONTAINER (vbox), calwin->priv->locations_list);
 
-	//gtk_box_pack_start (GTK_BOX (vbox), calwin->priv->locations_list, TRUE, FALSE, 0);
+	/* gtk_box_pack_start (GTK_BOX (vbox), calwin->priv->locations_list, TRUE, FALSE, 0); */
 }
 
 static void
@@ -403,6 +403,8 @@ calendar_window_dispose (GObject *object)
 
 	calwin = CALENDAR_WINDOW (object);
 
+	g_clear_pointer (&calwin->priv->prefs_path, g_free);
+
 	if (calwin->priv->settings)
 		g_object_unref (calwin->priv->settings);
 	calwin->priv->settings = NULL;
@@ -524,7 +526,7 @@ calendar_window_set_invert_order (CalendarWindow *calwin,
 		return;
 
 	calwin->priv->invert_order = invert_order;
-	//FIXME: update the order of the content of the window
+	/* FIXME: update the order of the content of the window */
 
 	g_object_notify (G_OBJECT (calwin), "invert-order");
 }
