@@ -100,7 +100,7 @@ clock_map_class_init (ClockMapClass *this_class)
 	 * The map widget emits this signal when it needs to know which
 	 * locations to display.
 	 *
-	 * Returns: the handler should return a (GList *) of (ClockLocation *).
+	 * Returns: the handler should return a (GSList *) of (ClockLocation *).
 	 * The map widget will not modify this list, so the caller should keep
 	 * it alive.
 	 */
@@ -429,8 +429,7 @@ static void
 clock_map_place_locations (ClockMap *this)
 {
         ClockMapPrivate *priv = clock_map_get_instance_private (this);
-        GList *locs;
-        ClockLocation *loc;
+        GSList *locs;
 
         if (priv->location_map_pixbuf) {
                 g_object_unref (priv->location_map_pixbuf);
@@ -443,10 +442,8 @@ clock_map_place_locations (ClockMap *this)
 	g_signal_emit (this, signals[NEED_LOCATIONS], 0, &locs);
 
         while (locs) {
-                loc = CLOCK_LOCATION (locs->data);
-
+                ClockLocation *loc = CLOCK_LOCATION (locs->data);
                 clock_map_place_location (this, loc, FALSE);
-
                 locs = locs->next;
         }
 
