@@ -27,7 +27,6 @@
 
 #include <config.h>
 
-#include <math.h>
 #include <string.h>
 #include <time.h>
 
@@ -212,7 +211,8 @@ static void name_value_changed(GtkEntry* entry, FishApplet* fish)
 }
 
 static void image_value_changed(GtkFileChooser* chooser, FishApplet* fish)
-{	char *path;
+{
+	char *path;
 	char *image;
 	char *path_gsettings;
 
@@ -261,7 +261,6 @@ static void command_value_changed(GtkEntry* entry, FishApplet *fish)
 	    !strcmp  (text, "uptime")  ||
 	    !strncmp (text, "tail ", 5)) {
 		static gboolean message_given = FALSE;
-		char       *message;
 		const char *warning_format =
 				_("Warning:  The command "
 				  "appears to be something actually useful.\n"
@@ -272,13 +271,10 @@ static void command_value_changed(GtkEntry* entry, FishApplet *fish)
 				   "which would make the applet "
 				   "\"practical\" or useful.");
 
-		if ( ! message_given) {
-			message = g_strdup_printf (warning_format, fish->name);
-
+		if (!message_given) {
+			char *message = g_strdup_printf (warning_format, fish->name);
 			something_fishy_going_on (fish, message);
-
 			g_free (message);
-
 			message_given = TRUE;
 		}
 	}
@@ -472,7 +468,7 @@ static void display_preferences_dialog(GtkAction* action, FishApplet* fish)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (fish->frames_spin),
 				   fish->n_frames);
 
-	g_signal_connect (fish->frames_spin, "value_changed",
+	g_signal_connect (fish->frames_spin, "value-changed",
 			  G_CALLBACK (n_frames_value_changed), fish);
 
 	setup_sensitivity (fish, builder,
@@ -484,7 +480,7 @@ static void display_preferences_dialog(GtkAction* action, FishApplet* fish)
 	fish->speed_spin = GTK_WIDGET (gtk_builder_get_object (builder, "speed_spin"));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (fish->speed_spin), fish->speed);
 
-	g_signal_connect (fish->speed_spin, "value_changed",
+	g_signal_connect (fish->speed_spin, "value-changed",
 			  G_CALLBACK (speed_value_changed), fish);
 
 	setup_sensitivity (fish, builder,
@@ -561,12 +557,12 @@ static void display_about_dialog(GtkAction* action, FishApplet* fish)
 		"comments", descr,
 		"copyright", _("Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.\n"
 		               "Copyright \xc2\xa9 2002-2005 Vincent Untz\n"
-		               "Copyright \xc2\xa9 2012-2020 MATE developers"),
+		               "Copyright \xc2\xa9 2012-2021 MATE developers"),
 		"documenters", documenters,
 		"logo-icon-name", FISH_ICON,
 		"translator-credits", _("translator-credits"),
-		"version", VERSION, // "3.4.7.4ac19"
-		"website", "http://mate-desktop.org/",
+		"version", VERSION, /* "3.4.7.4ac19" */
+		"website", PACKAGE_URL,
 		NULL);
 
 	g_free(descr);
@@ -729,7 +725,6 @@ static void clear_fortune_text(FishApplet* fish)
 static gboolean fish_read_output(GIOChannel* source, GIOCondition condition, gpointer data)
 {
 	char        output[4096];
-	char       *utf8_output;
 	gsize       bytes_read;
 	GError     *error = NULL;
 	GIOStatus   status;
@@ -763,6 +758,8 @@ static gboolean fish_read_output(GIOChannel* source, GIOCondition condition, gpo
 		return TRUE;
 
 	if (bytes_read > 0) {
+		char *utf8_output;
+
 		/* The output is not guarantied to be in UTF-8 format, most
 		 * likely it's just in ASCII-7 or in the user locale
 		 */
@@ -1002,11 +999,12 @@ static void name_changed_notify(GSettings* settings, gchar* key, FishApplet* fis
 
 	value = g_settings_get_string (settings, key);
 
-	if (!value [0] || (fish->name && !strcmp (fish->name, value)))
+	if (!value || *value == '\0' || (fish->name && !strcmp (fish->name, value))) {
+		g_free (value);
 		return;
+	}
 
-	if (fish->name)
-		g_free (fish->name);
+	g_free (fish->name);
 	fish->name = g_strdup (value);
 
 	update_fortune_dialog (fish);
@@ -1017,8 +1015,7 @@ static void name_changed_notify(GSettings* settings, gchar* key, FishApplet* fis
 	    strcmp (gtk_entry_get_text (GTK_ENTRY (fish->name_entry)), fish->name))
 		gtk_entry_set_text (GTK_ENTRY (fish->name_entry), fish->name);
 
-	if (value)
-		g_free (value);
+	g_free (value);
 }
 
 static void image_changed_notify(GSettings* settings, gchar* key, FishApplet* fish)
@@ -1027,11 +1024,12 @@ static void image_changed_notify(GSettings* settings, gchar* key, FishApplet* fi
 
 	value = g_settings_get_string (settings, key);
 
-	if (!value [0] || (fish->image && !strcmp (fish->image, value)))
+	if (!value || *value == '\0' || (fish->image && !strcmp (fish->image, value))) {
+		g_free (value);
 		return;
+	}
 
-	if (fish->image)
-		g_free (fish->image);
+	g_free (fish->image);
 	fish->image = g_strdup (value);
 
 	load_fish_image (fish);
@@ -1051,8 +1049,7 @@ static void image_changed_notify(GSettings* settings, gchar* key, FishApplet* fi
 		g_free (path_chooser);
 	}
 
-	if (value)
-		g_free (value);
+	g_free (value);
 }
 
 static void command_changed_notify(GSettings* settings, gchar* key, FishApplet* fish)
@@ -1061,19 +1058,19 @@ static void command_changed_notify(GSettings* settings, gchar* key, FishApplet*
 
 	value = g_settings_get_string (settings, key);
 
-	if (fish->command && !strcmp (fish->command, value))
+	if (!value || *value == '\0' || (fish->command && !strcmp (fish->command, value))) {
+		g_free (value);
 		return;
+	}
 
-	if (fish->command)
-		g_free (fish->command);
+	g_free (fish->command);
 	fish->command = g_strdup (value);
 
 	if (fish->command_entry &&
 	    strcmp (gtk_entry_get_text (GTK_ENTRY (fish->command_entry)), fish->command))
 		gtk_entry_set_text (GTK_ENTRY (fish->command_entry), fish->command);
 
-	if (value)
-		g_free (value);
+	g_free (value);
 }
 
 static void n_frames_changed_notify(GSettings* settings, gchar* key, FishApplet* fish)
@@ -1438,16 +1435,16 @@ static void update_pixmap(FishApplet* fish)
 	if (fish->april_fools) {
 		cairo_matrix_translate (&matrix,
 					pixbuf_width - 1, pixbuf_height - 1);
-		cairo_matrix_rotate (&matrix, M_PI);
+		cairo_matrix_rotate (&matrix, G_PI);
 	}
 
 	if (rotate) {
 		if (fish->orientation == MATE_PANEL_APPLET_ORIENT_RIGHT) {
 			cairo_matrix_translate (&matrix, pixbuf_width - 1, 0);
-			cairo_matrix_rotate (&matrix, M_PI * 0.5);
+			cairo_matrix_rotate (&matrix, G_PI_2);
 		} else {
 			cairo_matrix_translate (&matrix, 0, pixbuf_height - 1);
-			cairo_matrix_rotate (&matrix, M_PI * 1.5);
+			cairo_matrix_rotate (&matrix, G_PI * 1.5);
 		}
 		cairo_matrix_scale (&matrix,
 				    (double) (pixbuf_height - 1) / width,
@@ -1796,16 +1793,13 @@ static void fish_applet_dispose (GObject *object)
 		g_object_unref (fish->lockdown_settings);
 	fish->lockdown_settings = NULL;
 
-	if (fish->name)
-		g_free (fish->name);
+	g_free (fish->name);
 	fish->name = NULL;
 
-	if (fish->image)
-		g_free (fish->image);
+	g_free (fish->image);
 	fish->image = NULL;
 
-	if (fish->command)
-		g_free (fish->command);
+	g_free (fish->command);
 	fish->command = NULL;
 
 	if (fish->surface)
@@ -1885,8 +1879,6 @@ static void fish_applet_instance_init(FishApplet* fish, FishAppletClass* klass)
 	fish->april_fools = FALSE;
 
 	mate_panel_applet_set_flags (MATE_PANEL_APPLET (fish), MATE_PANEL_APPLET_EXPAND_MINOR);
-
-	mate_panel_applet_set_background_widget(MATE_PANEL_APPLET(fish), GTK_WIDGET(fish));
 }
 
 static void fish_applet_class_init(FishAppletClass* klass)
