@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
+#include <config.h>
 
 #include <math.h>
 
@@ -59,6 +59,7 @@ struct _SnItemV0
   gchar         *title;
   gint32         window_id;
   gchar         *icon_name;
+  gchar         *label;
   SnIconPixmap **icon_pixmap;
   gchar         *overlay_icon_name;
   SnIconPixmap **overlay_icon_pixmap;
@@ -83,7 +84,7 @@ enum
   LAST_PROP
 };
 
-static GParamSpec *properties[LAST_PROP] = { NULL };
+static GParamSpec *obj_properties[LAST_PROP] = { NULL };
 
 G_DEFINE_TYPE (SnItemV0, sn_item_v0, SN_TYPE_ITEM)
 
@@ -329,6 +330,7 @@ update (SnItemV0 *v0)
       gtk_widget_set_tooltip_markup (GTK_WIDGET (v0), NULL);
     }
 
+  gtk_button_set_label (GTK_BUTTON (v0), v0->label);
   accessible = gtk_widget_get_accessible (GTK_WIDGET (v0));
 
   if (v0->title != NULL && *v0->title != '\0')
@@ -847,6 +849,21 @@ new_status_cb (SnItemV0 *v0,
   queue_update (v0);
 }
 
+static void
+new_label_cb (SnItemV0 *v0,
+              GVariant *parameters)
+{
+  GVariant *variant;
+
+  variant = g_variant_get_child_value (parameters, 0);
+
+  g_free (v0->label);
+  v0->label = g_variant_dup_string (variant, NULL);
+  g_variant_unref (variant);
+
+  queue_update (v0);
+}
+
 static void
 new_icon_theme_path_cb (SnItemV0 *v0,
                         GVariant *parameters)
@@ -905,6 +922,8 @@ g_signal_cb (GDBusProxy *proxy,
     new_status_cb (v0, parameters);
   else if (g_strcmp0 (signal_name, "NewIconThemePath") == 0)
     new_icon_theme_path_cb (v0, parameters);
+  else if (g_strcmp0 (signal_name, "XAyatanaNewLabel") == 0)
+    new_label_cb (v0, parameters);
   else
     g_debug ("signal '%s' not handled!", signal_name);
 }
@@ -975,6 +994,8 @@ get_all_cb (GObject      *source_object,
         v0->menu = g_variant_dup_string (value, NULL);
       else if (g_strcmp0 (key, "ItemIsMenu") == 0)
         v0->item_is_menu = g_variant_get_boolean (value);
+      else if (g_strcmp0 (key, "XAyatanaLabel") == 0)
+        v0->label = g_variant_dup_string (value, NULL);
       else
         g_debug ("property '%s' not handled!", key);
 
@@ -1111,6 +1132,7 @@ sn_item_v0_finalize (GObject *object)
 
   g_clear_pointer (&v0->title, g_free);
   g_clear_pointer (&v0->icon_name, g_free);
+  g_clear_pointer (&v0->label, g_free);
   g_clear_pointer (&v0->icon_pixmap, icon_pixmap_free);
   g_clear_pointer (&v0->overlay_icon_name, g_free);
   g_clear_pointer (&v0->overlay_icon_pixmap, icon_pixmap_free);
@@ -1345,15 +1367,15 @@ sn_item_v0_set_property (GObject      *object,
 static void
 install_properties (GObjectClass *object_class)
 {
-  properties[PROP_ICON_SIZE] =
+  obj_properties[PROP_ICON_SIZE] =
     g_param_spec_int ("icon-size", "Icon size", "Icon size", 0, G_MAXINT, 16,
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  properties[PROP_ICON_PADDING] =
+  obj_properties[PROP_ICON_PADDING] =
     g_param_spec_int ("icon-padding", "Icon padding", "Icon padding", 0,
                       G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
-  g_object_class_install_properties (object_class, LAST_PROP, properties);
+  g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 }
 
 static void
@@ -1395,7 +1417,7 @@ sn_item_v0_init (SnItemV0 *v0)
   v0->icon_size = 16;
   v0->effective_icon_size = 0;
   v0->image = gtk_image_new ();
-  gtk_container_add (GTK_CONTAINER (v0), v0->image);
+  gtk_button_set_image (GTK_BUTTON (v0), v0->image);
   gtk_widget_show (v0->image);
 }
 
@@ -1465,7 +1487,7 @@ sn_item_v0_set_icon_size (SnItemV0 *v0,
   if (v0->icon_size != size)
     {
       v0->icon_size = size;
-      g_object_notify_by_pspec (G_OBJECT (v0), properties[PROP_ICON_SIZE]);
+      g_object_notify_by_pspec (G_OBJECT (v0), obj_properties[PROP_ICON_SIZE]);
 
       if (v0->id != NULL)
         queue_update (v0);
