@@ -22,20 +22,18 @@
  */
 
 #ifdef HAVE_CONFIG_H
-	#include <config.h>
-#endif
-
-#ifndef HAVE_X11
-#error file should only be built when HAVE_X11 is enabled
+#include <config.h>
 #endif
 
 #include <glib/gi18n.h>
 
 #include <gtk/gtk.h>
-#include <gdk/gdkx.h>
 
+#ifdef HAVE_X11
+#include <gdk/gdkx.h>
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
+#endif
 
 #include "wncklet.h"
 #include "showdesktop.h"
@@ -361,14 +359,20 @@ static void show_desktop_applet_realized(MatePanelApplet* applet, gpointer data)
 
 	sdd = (ShowDesktopData*) data;
 
-	if (sdd->wnck_screen != NULL)
-		g_signal_handlers_disconnect_by_func(sdd->wnck_screen, show_desktop_changed_callback, sdd);
-
 	if (sdd->icon_theme != NULL)
 		g_signal_handlers_disconnect_by_func(sdd->icon_theme, theme_changed_callback, sdd);
 
 	screen = gtk_widget_get_screen(sdd->applet);
-	sdd->wnck_screen = wnck_screen_get(gdk_x11_screen_get_screen_number (screen));
+
+	if (sdd->wnck_screen != NULL)
+		g_signal_handlers_disconnect_by_func(sdd->wnck_screen, show_desktop_changed_callback, sdd);
+
+	sdd->wnck_screen = NULL;
+
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+		sdd->wnck_screen = wnck_screen_get(gdk_x11_screen_get_screen_number (screen));
+#endif /* HAVE_X11 */
 
 	if (sdd->wnck_screen != NULL)
 		wncklet_connect_while_alive(sdd->wnck_screen, "showing_desktop_changed", G_CALLBACK(show_desktop_changed_callback), sdd, sdd->applet);
@@ -454,8 +458,6 @@ gboolean show_desktop_applet_fill(MatePanelApplet* applet)
 	   initial oriantation, and we get that during the _add call */
 	g_signal_connect(G_OBJECT (sdd->applet), "change_orient", G_CALLBACK (applet_change_orient), sdd);
 
-	mate_panel_applet_set_background_widget(MATE_PANEL_APPLET (sdd->applet), GTK_WIDGET(sdd->applet));
-
 	action_group = gtk_action_group_new("ShowDesktop Applet Actions");
 	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions(action_group, show_desktop_menu_actions, G_N_ELEMENTS (show_desktop_menu_actions), sdd);
@@ -502,19 +504,34 @@ static void display_about_dialog(GtkAction* action, ShowDesktopData* sdd)
 		"comments", _("This button lets you hide all windows and show the desktop."),
 		"copyright", _("Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
 		               "Copyright \xc2\xa9 2011 Perberos\n"
-                               "Copyright \xc2\xa9 2012-2020 MATE developers"),
+                               "Copyright \xc2\xa9 2012-2021 MATE developers"),
 		"documenters", documenters,
 		"icon-name", SHOW_DESKTOP_ICON,
 		"logo-icon-name", SHOW_DESKTOP_ICON,
 		"translator-credits", _("translator-credits"),
 		"version", VERSION,
-		"website", "http://www.mate-desktop.org/",
+		"website", PACKAGE_URL,
 		NULL);
 }
 
 static void button_toggled_callback(GtkWidget* button, ShowDesktopData* sdd)
 {
-	if (!gdk_x11_screen_supports_net_wm_hint(gtk_widget_get_screen(button), gdk_atom_intern("_NET_SHOWING_DESKTOP", FALSE)))
+	gboolean can_show_desktop;
+
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+	{
+		can_show_desktop = gdk_x11_screen_supports_net_wm_hint(gtk_widget_get_screen(button),
+								       gdk_atom_intern("_NET_SHOWING_DESKTOP",
+								       FALSE));
+	}
+	else
+#endif
+	{ /* not using X11 */
+		can_show_desktop = FALSE;
+	}
+
+	if (!can_show_desktop)
 	{
 		static GtkWidget* dialog = NULL;
 
@@ -527,7 +544,11 @@ static void button_toggled_callback(GtkWidget* button, ShowDesktopData* sdd)
 			return;
 		}
 
-		dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, _("Your window manager does not support the show desktop button, or you are not running a window manager."));
+		dialog = gtk_message_dialog_new(NULL,
+						GTK_DIALOG_MODAL,
+						GTK_MESSAGE_ERROR,
+						GTK_BUTTONS_CLOSE,
+						_("Your window manager does not support the show desktop button, or you are not running a window manager."));
 
 		g_object_add_weak_pointer(G_OBJECT(dialog), (gpointer) &dialog);
 
@@ -540,16 +561,20 @@ static void button_toggled_callback(GtkWidget* button, ShowDesktopData* sdd)
 		return;
 	}
 
+#ifdef HAVE_X11
 	if (sdd->wnck_screen != NULL)
 		wnck_screen_toggle_showing_desktop(sdd->wnck_screen, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
+#endif /* HAVE_X11 */
 
 	update_button_display (sdd);
 }
 
 static void show_desktop_changed_callback(WnckScreen* screen, ShowDesktopData* sdd)
 {
+#ifdef HAVE_X11
 	if (sdd->wnck_screen != NULL)
 		sdd->showing_desktop = wnck_screen_get_showing_desktop(sdd->wnck_screen);
+#endif /* HAVE_X11 */
 
 	update_button_state (sdd);
 }
