@@ -21,9 +21,17 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
+
+#ifdef HAVE_X11
+#include <gdk/gdkx.h>
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
-#include <gio/gio.h>
+#endif /* HAVE_X11 */
+
+#ifdef HAVE_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif /* HAVE_WAYLAND */
 
 #include <libmate-desktop/mate-gsettings.h>
 
@@ -50,6 +58,7 @@ typedef enum {
 	PAGER_WM_METACITY,
 	PAGER_WM_COMPIZ,
 	PAGER_WM_I3,
+	PAGER_WM_XMONAD,
 	PAGER_WM_UNKNOWN
 } PagerWM;
 
@@ -79,7 +88,7 @@ typedef struct {
 
 	GtkOrientation orientation;
 	int n_rows;				/* for vertical layout this is cols */
-	WnckPagerDisplayMode display_mode;
+	gboolean display_names;			/* if to display names or content */
 	gboolean display_all;
 	gboolean wrap_workspaces;
 
@@ -91,20 +100,35 @@ static void display_help_dialog(GtkAction* action, PagerData* pager);
 static void display_about_dialog(GtkAction* action, PagerData* pager);
 static void destroy_pager(GtkWidget* widget, PagerData* pager);
 
+#ifdef HAVE_X11
+static void pager_update_wnck(PagerData* pager, WnckPager* wnck_pager)
+{
+	WnckPagerDisplayMode display_mode = WNCK_PAGER_DISPLAY_CONTENT;
+
+	if (pager->display_names && (
+		pager->wm == PAGER_WM_MARCO ||
+		pager->wm == PAGER_WM_METACITY ||
+		pager->wm == PAGER_WM_I3 ||
+		pager->wm == PAGER_WM_XMONAD))
+	{
+		display_mode = WNCK_PAGER_DISPLAY_NAME;
+	}
+
+	wnck_pager_set_orientation(wnck_pager, pager->orientation);
+	wnck_pager_set_n_rows(wnck_pager, pager->n_rows);
+	wnck_pager_set_show_all(wnck_pager, pager->display_all);
+	wnck_pager_set_display_mode(wnck_pager, display_mode);
+}
+#endif /* HAVE_X11 */
+
 static void pager_update(PagerData* pager)
 {
-	wnck_pager_set_orientation(WNCK_PAGER(pager->pager), pager->orientation);
-	wnck_pager_set_n_rows(WNCK_PAGER(pager->pager), pager->n_rows);
-	wnck_pager_set_show_all(WNCK_PAGER(pager->pager), pager->display_all);
-
-	if (pager->wm == PAGER_WM_MARCO)
-		wnck_pager_set_display_mode(WNCK_PAGER(pager->pager), pager->display_mode);
-	else if (pager->wm == PAGER_WM_METACITY)
-		wnck_pager_set_display_mode(WNCK_PAGER(pager->pager), pager->display_mode);
-	else if (pager->wm == PAGER_WM_I3)
-		wnck_pager_set_display_mode(WNCK_PAGER(pager->pager), pager->display_mode);
-	else
-		wnck_pager_set_display_mode(WNCK_PAGER(pager->pager), WNCK_PAGER_DISPLAY_CONTENT);
+#ifdef HAVE_X11
+	if (WNCK_IS_PAGER(pager->pager))
+	{
+		pager_update_wnck(pager, WNCK_PAGER(pager->pager));
+	}
+#endif /* HAVE_X11 */
 }
 
 static void update_properties_for_wm(PagerData* pager)
@@ -149,6 +173,20 @@ static void update_properties_for_wm(PagerData* pager)
 			if (pager->cell)
 				g_object_set (pager->cell, "editable", FALSE, NULL);
 			break;
+		case PAGER_WM_XMONAD:
+			if (pager->workspaces_frame)
+				gtk_widget_show(pager->workspaces_frame);
+			if (pager->num_workspaces_spin)
+				gtk_widget_set_sensitive(pager->num_workspaces_spin, FALSE);
+			if (pager->workspace_names_label)
+				gtk_widget_hide(pager->workspace_names_label);
+			if (pager->workspace_names_scroll)
+				gtk_widget_hide(pager->workspace_names_scroll);
+			if (pager->display_workspaces_toggle)
+				gtk_widget_show(pager->display_workspaces_toggle);
+			if (pager->cell)
+				g_object_set (pager->cell, "editable", FALSE, NULL);
+			break;
 		case PAGER_WM_COMPIZ:
 			if (pager->workspaces_frame)
 				gtk_widget_show(pager->workspaces_frame);
@@ -178,9 +216,13 @@ static void update_properties_for_wm(PagerData* pager)
 
 static void window_manager_changed(WnckScreen* screen, PagerData* pager)
 {
-	const char *wm_name;
+#ifdef HAVE_X11
+	const char *wm_name = NULL;
 
-	wm_name = wnck_screen_get_window_manager_name(screen);
+	if (pager->screen)
+	{
+		wm_name = wnck_screen_get_window_manager_name (pager->screen);
+	}
 
 	if (!wm_name)
 		pager->wm = PAGER_WM_UNKNOWN;
@@ -190,10 +232,15 @@ static void window_manager_changed(WnckScreen* screen, PagerData* pager)
 		pager->wm = PAGER_WM_METACITY;
 	else if (strcmp(wm_name, "i3") == 0)
 		pager->wm = PAGER_WM_I3;
+	else if (strcmp(wm_name, "xmonad") == 0)
+		pager->wm = PAGER_WM_XMONAD;
 	else if (strcmp(wm_name, "Compiz") == 0)
 		pager->wm = PAGER_WM_COMPIZ;
 	else
 		pager->wm = PAGER_WM_UNKNOWN;
+#else
+	pager->wm = PAGER_WM_UNKNOWN;
+#endif /* HAVE_X11 */
 
 	update_properties_for_wm(pager);
 	pager_update(pager);
@@ -201,15 +248,22 @@ static void window_manager_changed(WnckScreen* screen, PagerData* pager)
 
 static void applet_realized(MatePanelApplet* applet, PagerData* pager)
 {
-	pager->screen = wncklet_get_screen(GTK_WIDGET(applet));
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+	{
+		pager->screen = wncklet_get_screen(GTK_WIDGET(applet));
+		wncklet_connect_while_alive(pager->screen, "window_manager_changed", G_CALLBACK(window_manager_changed), pager, pager->applet);
+	}
+#endif /* HAVE_X11 */
 
 	window_manager_changed(pager->screen, pager);
-	wncklet_connect_while_alive(pager->screen, "window_manager_changed", G_CALLBACK(window_manager_changed), pager, pager->applet);
 }
 
 static void applet_unrealized(MatePanelApplet* applet, PagerData* pager)
 {
+#ifdef HAVE_X11
 	pager->screen = NULL;
+#endif /* HAVE_X11 */
 	pager->wm = PAGER_WM_UNKNOWN;
 }
 
@@ -248,8 +302,13 @@ static void applet_change_background(MatePanelApplet* applet, MatePanelAppletBac
         gtk_style_context_set_path (new_context, gtk_widget_get_path (GTK_WIDGET (pager->pager)));
         g_object_unref (new_context);
 
-        wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
-                type == PANEL_NO_BACKGROUND ? GTK_SHADOW_NONE : GTK_SHADOW_IN);
+#ifdef HAVE_X11
+	if (WNCK_IS_PAGER(pager->pager))
+	{
+		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
+		        type == PANEL_NO_BACKGROUND ? GTK_SHADOW_NONE : GTK_SHADOW_IN);
+	}
+#endif /* HAVE_X11 */
 }
 
 static void applet_style_updated (MatePanelApplet *applet, GtkStyleContext *context)
@@ -282,11 +341,13 @@ static void applet_style_updated (MatePanelApplet *applet, GtkStyleContext *cont
  */
 static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, PagerData* pager)
 {
+#ifdef HAVE_X11
 	GdkScrollDirection absolute_direction;
 	int index;
 	int n_workspaces;
 	int n_columns;
 	int in_last_row;
+#endif /* HAVE_X11 */
 
 	if (event->type != GDK_SCROLL)
 		return FALSE;
@@ -294,8 +355,18 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 	if (event->direction == GDK_SCROLL_SMOOTH)
 		return FALSE;
 
-	index = wnck_workspace_get_number(wnck_screen_get_active_workspace(pager->screen));
-	n_workspaces = wnck_screen_get_workspace_count(pager->screen);
+#ifdef HAVE_X11
+	if (pager->screen)
+	{
+		index = wnck_workspace_get_number(wnck_screen_get_active_workspace(pager->screen));
+		n_workspaces = wnck_screen_get_workspace_count(pager->screen);
+	}
+	else
+	{
+		index = 0;
+		n_workspaces = 1;
+	}
+
 	n_columns = n_workspaces / pager->n_rows;
 
 	if (n_workspaces % pager->n_rows != 0)
@@ -381,7 +452,11 @@ static gboolean applet_scroll(MatePanelApplet* applet, GdkEventScroll* event, Pa
 			break;
 	}
 
-	wnck_workspace_activate(wnck_screen_get_workspace(pager->screen, index), event->time);
+	if (pager->screen)
+	{
+		wnck_workspace_activate(wnck_screen_get_workspace(pager->screen, index), event->time);
+	}
+#endif /* HAVE_X11 */
 
 	return TRUE;
 }
@@ -434,14 +509,7 @@ static void display_workspace_names_changed(GSettings* settings, gchar* key, Pag
 
 	value = g_settings_get_boolean (settings, key);
 
-	if (value)
-	{
-		pager->display_mode = WNCK_PAGER_DISPLAY_NAME;
-	}
-	else
-	{
-		pager->display_mode = WNCK_PAGER_DISPLAY_CONTENT;
-	}
+	pager->display_names = g_settings_get_boolean (settings, key);
 
 	pager_update(pager);
 
@@ -521,7 +589,6 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 {
 	PagerData* pager;
 	GtkActionGroup* action_group;
-	gboolean display_names;
 
 	pager = g_new0(PagerData, 1);
 
@@ -535,19 +602,10 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 
 	pager->n_rows = CLAMP(pager->n_rows, 1, MAX_REASONABLE_ROWS);
 
-	display_names = g_settings_get_boolean(pager->settings, "display-workspace-names");
+	pager->display_names = g_settings_get_boolean(pager->settings, "display-workspace-names");
 
 	pager->wrap_workspaces = g_settings_get_boolean(pager->settings, "wrap-workspaces");
 
-	if (display_names)
-	{
-		pager->display_mode = WNCK_PAGER_DISPLAY_NAME;
-	}
-	else
-	{
-		pager->display_mode = WNCK_PAGER_DISPLAY_CONTENT;
-	}
-
 	pager->display_all = g_settings_get_boolean(pager->settings, "display-all-workspaces");
 
 	switch (mate_panel_applet_get_orient(applet))
@@ -563,10 +621,28 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 			break;
 	}
 
-	pager->pager = wnck_pager_new();
-	pager->screen = NULL;
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+	{
+		pager->pager = wnck_pager_new();
+		wnck_pager_set_shadow_type(WNCK_PAGER(pager->pager), GTK_SHADOW_IN);
+	}
+	else
+#endif /* HAVE_X11 */
+
+#ifdef HAVE_WAYLAND
+	if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()))
+	{
+		pager->pager = gtk_label_new ("[Pager not supported on Wayland]");
+	}
+	else
+#endif /* HAVE_WAYLAND */
+
+	{
+		pager->pager = gtk_label_new ("[Pager not supported on this platform]");
+	}
+
 	pager->wm = PAGER_WM_UNKNOWN;
-	wnck_pager_set_shadow_type(WNCK_PAGER(pager->pager), GTK_SHADOW_IN);
 
 	GtkStyleContext *context;
 	context = gtk_widget_get_style_context (GTK_WIDGET (applet));
@@ -590,8 +666,6 @@ gboolean workspace_switcher_applet_fill(MatePanelApplet* applet)
 	gtk_widget_show(pager->pager);
 	gtk_widget_show(pager->applet);
 
-	mate_panel_applet_set_background_widget(MATE_PANEL_APPLET(pager->applet), GTK_WIDGET(pager->applet));
-
 	action_group = gtk_action_group_new("WorkspaceSwitcher Applet Actions");
 	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions(action_group, pager_menu_actions, G_N_ELEMENTS(pager_menu_actions), pager);
@@ -641,13 +715,13 @@ static void display_about_dialog(GtkAction* action, PagerData* pager)
 		"comments", _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
 		"copyright", _("Copyright \xc2\xa9 2002 Red Hat, Inc.\n"
                                "Copyright \xc2\xa9 2011 Perberos\n"
-                               "Copyright \xc2\xa9 2012-2020 MATE developers"),
+                               "Copyright \xc2\xa9 2012-2021 MATE developers"),
 		"documenters", documenters,
 		"icon-name", WORKSPACE_SWITCHER_ICON,
 		"logo-icon-name", WORKSPACE_SWITCHER_ICON,
 		"translator-credits", _("translator-credits"),
 		"version", VERSION,
-		"website", "http://www.mate-desktop.org/",
+		"website", PACKAGE_URL,
 		NULL);
 }
 
@@ -671,16 +745,34 @@ static void num_rows_value_changed(GtkSpinButton* button, PagerData* pager)
 	g_settings_set_int(pager->settings, "num-rows", gtk_spin_button_get_value_as_int(button));
 }
 
+static const char *get_workspace_name(PagerData* pager, int workspace_index)
+{
+#ifdef HAVE_X11
+	if (pager->screen)
+	{
+		WnckWorkspace *workspace = wnck_screen_get_workspace(pager->screen, workspace_index);
+		return wnck_workspace_get_name(workspace);
+	}
+#endif /* HAVE_X11 */
+
+	/* Fallback */
+	return "workspace";
+}
+
 static void update_workspaces_model(PagerData* pager)
 {
-	int nr_ws, i;
-	WnckWorkspace* workspace;
-	GtkTreeIter iter;
+	int nr_ws = 1;
 
-	nr_ws = wnck_screen_get_workspace_count(pager->screen);
+#ifdef HAVE_X11
+	if (pager->screen)
+		nr_ws = wnck_screen_get_workspace_count (pager->screen);
+#endif /* HAVE_X11 */
 
 	if (pager->properties_dialog)
 	{
+		int i;
+		GtkTreeIter iter;
+
 		if (nr_ws != gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pager->num_workspaces_spin)))
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_workspaces_spin), nr_ws);
 
@@ -688,21 +780,21 @@ static void update_workspaces_model(PagerData* pager)
 
 		for (i = 0; i < nr_ws; i++)
 		{
-			workspace = wnck_screen_get_workspace(pager->screen, i);
 			gtk_list_store_append(pager->workspaces_store, &iter);
-			gtk_list_store_set(pager->workspaces_store, &iter, 0, wnck_workspace_get_name(workspace), -1);
+			const char* name = get_workspace_name(pager, i);
+			gtk_list_store_set(pager->workspaces_store, &iter, 0, name, -1);
 		}
 	}
 }
 
+#ifdef HAVE_X11
 static void workspace_renamed(WnckWorkspace* space, PagerData* pager)
 {
-	int i;
 	GtkTreeIter iter;
 
-	i = wnck_workspace_get_number(space);
+	g_return_if_fail(WNCK_IS_WORKSPACE(space));
 
-	if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(pager->workspaces_store), &iter, NULL, i))
+	if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pager->workspaces_store), &iter, NULL, wnck_workspace_get_number (space)))
 		gtk_list_store_set(pager->workspaces_store, &iter, 0, wnck_workspace_get_name(space), -1);
 }
 
@@ -720,10 +812,19 @@ static void workspace_destroyed(WnckScreen* screen, WnckWorkspace* space, PagerD
 	g_return_if_fail(WNCK_IS_SCREEN(screen));
 	update_workspaces_model(pager);
 }
+#endif /* HAVE_X11 */
 
-static void num_workspaces_value_changed(GtkSpinButton* button, PagerData* pager)
+static void
+on_num_workspaces_value_changed (GtkSpinButton *button,
+                                 PagerData     *pager)
 {
-	wnck_screen_change_workspace_count(pager->screen, gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(pager->num_workspaces_spin)));
+#ifdef HAVE_X11
+	if (pager->screen)
+	{
+		int workspace_count = gtk_spin_button_get_value_as_int (button);
+		wnck_screen_change_workspace_count(pager->screen, workspace_count);
+	}
+#endif /* HAVE_X11 */
 }
 
 static gboolean workspaces_tree_focused_out(GtkTreeView* treeview, GdkEventFocus* event, PagerData* pager)
@@ -737,28 +838,33 @@ static gboolean workspaces_tree_focused_out(GtkTreeView* treeview, GdkEventFocus
 
 static void workspace_name_edited(GtkCellRendererText* cell_renderer_text, const gchar* path, const gchar* new_text, PagerData* pager)
 {
-	const gint* indices;
-	WnckWorkspace* workspace;
-	GtkTreePath* p;
-
-	p = gtk_tree_path_new_from_string(path);
-	indices = gtk_tree_path_get_indices(p);
-	workspace = wnck_screen_get_workspace(pager->screen, indices[0]);
-
-	if (workspace != NULL)
+#ifdef HAVE_X11
+	if (pager->screen)
 	{
-		gchar* temp_name = g_strdup(new_text);
+		const gint* indices;
+		WnckWorkspace* workspace;
+		GtkTreePath* p;
 
-		wnck_workspace_change_name(workspace, g_strstrip(temp_name));
+		p = gtk_tree_path_new_from_string(path);
+		indices = gtk_tree_path_get_indices(p);
+		workspace = wnck_screen_get_workspace(pager->screen, indices[0]);
 
-		g_free(temp_name);
+		if (workspace != NULL)
+		{
+			gchar* temp_name = g_strdup(new_text);
+
+			wnck_workspace_change_name(workspace, g_strstrip(temp_name));
+
+			g_free(temp_name);
+		}
+		else
+		{
+			g_warning("Edited name of workspace %d which no longer exists", indices[0]);
+		}
+
+		gtk_tree_path_free(p);
 	}
-	else
-	{
-		g_warning("Edited name of workspace %d which no longer exists", indices[0]);
-	}
-
-	gtk_tree_path_free(p);
+#endif
 }
 
 static void properties_dialog_destroyed(GtkWidget* widget, PagerData* pager)
@@ -855,7 +961,6 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	gboolean value;
 	GtkTreeViewColumn* column;
 	GtkCellRenderer* cell;
-	int nr_ws, i;
 	GSettings *marco_general_settings = NULL;
 	GSettings *marco_workspaces_settings = NULL;
 
@@ -907,14 +1012,7 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 
 	g_signal_connect(G_OBJECT(pager->display_workspaces_toggle), "toggled", (GCallback) display_workspace_names_toggled, pager);
 
-	if (pager->display_mode == WNCK_PAGER_DISPLAY_NAME)
-	{
-		value = TRUE;
-	}
-	else
-	{
-		value = FALSE;
-	}
+	value = pager->display_names;
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pager->display_workspaces_toggle), value);
 
@@ -937,7 +1035,7 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	}
 
 	/* Num rows: */
-	g_signal_connect(G_OBJECT(pager->num_rows_spin), "value_changed", (GCallback) num_rows_value_changed, pager);
+	g_signal_connect (pager->num_rows_spin, "value-changed", G_CALLBACK (num_rows_value_changed), pager);
 
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_rows_spin), pager->n_rows);
 	gtk_label_set_text(GTK_LABEL(pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("rows") : _("columns"));
@@ -948,12 +1046,24 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 
 	g_signal_connect(WID("done_button"), "clicked", (GCallback) close_dialog, pager);
 
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_workspaces_spin), wnck_screen_get_workspace_count(pager->screen));
-	g_signal_connect(G_OBJECT(pager->num_workspaces_spin), "value_changed", (GCallback) num_workspaces_value_changed, pager);
+#ifdef HAVE_X11
+	if (pager->screen)
+	{
+		int i, nr_ws;
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(pager->num_workspaces_spin), wnck_screen_get_workspace_count(pager->screen));
+		wncklet_connect_while_alive(pager->screen, "workspace_created", G_CALLBACK(workspace_created), pager, pager->properties_dialog);
+		wncklet_connect_while_alive(pager->screen, "workspace_destroyed", G_CALLBACK(workspace_destroyed), pager, pager->properties_dialog);
 
-	wncklet_connect_while_alive(pager->screen, "workspace_created", G_CALLBACK(workspace_created), pager, pager->properties_dialog);
+		nr_ws = wnck_screen_get_workspace_count(pager->screen);
 
-	wncklet_connect_while_alive(pager->screen, "workspace_destroyed", G_CALLBACK(workspace_destroyed), pager, pager->properties_dialog);
+		for (i = 0; i < nr_ws; i++)
+		{
+			wncklet_connect_while_alive(G_OBJECT(wnck_screen_get_workspace(pager->screen, i)), "name_changed", G_CALLBACK(workspace_renamed), pager, pager->properties_dialog);
+		}
+	}
+#endif /* HAVE_X11 */
+
+	g_signal_connect (pager->num_workspaces_spin, "value-changed", G_CALLBACK (on_num_workspaces_value_changed), pager);
 
 	g_signal_connect(G_OBJECT(pager->workspaces_tree), "focus_out_event", (GCallback) workspaces_tree_focused_out, pager);
 
@@ -969,13 +1079,6 @@ static void setup_dialog(GtkBuilder* builder, PagerData* pager)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(pager->workspaces_tree), column);
 	g_signal_connect(cell, "edited", (GCallback) workspace_name_edited, pager);
 
-	nr_ws = wnck_screen_get_workspace_count(pager->screen);
-
-	for (i = 0; i < nr_ws; i++)
-	{
-		wncklet_connect_while_alive(G_OBJECT(wnck_screen_get_workspace(pager->screen, i)), "name_changed", G_CALLBACK(workspace_renamed), pager, pager->properties_dialog);
-	}
-
 	update_properties_for_wm(pager);
 }
 
