@@ -1,5 +1,6 @@
 /* Mate panel: general applet functionality
  * (C) 1997 the Free Software Foundation
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * Authors:  George Lebl
  *           Federico Mena
@@ -643,14 +644,11 @@ applet_show_menu (AppletInfo     *info,
 
 	g_return_if_fail (info != NULL);
 
-	panel_widget = mate_panel_applet_get_panel_widget (info);
-
-	if (info->menu == NULL)
-		info->menu = mate_panel_applet_create_menu (info);
-
-	if (info->menu == NULL)
+	if ((info->menu == NULL) && ((info->menu = mate_panel_applet_create_menu (info)) == NULL))
 		return;
 
+	panel_widget = mate_panel_applet_get_panel_widget (info);
+
 	mate_panel_applet_menu_set_recurse (GTK_MENU (info->menu),
 				       "menu_panel",
 				       panel_widget);
@@ -1142,9 +1140,9 @@ mate_panel_applet_save_position (AppletInfo *applet_info,
 		g_settings_set_boolean (applet_info->settings, PANEL_OBJECT_LOCKED_KEY, locked);
 
 	if (locked) {
-		// Until position calculations are refactored to fix the issue of the panel applets
-		// getting reordered on resolution changes...
-		// .. don't save position/right-stick on locked applets
+		/* Until position calculations are refactored to fix the issue of the panel applets
+		   getting reordered on resolution changes...
+		   .. don't save position/right-stick on locked applets */
 		return;
 	}
 
