@@ -1,5 +1,4 @@
 #include <config.h>
-#include <math.h>
 #include <string.h>
 
 #include <glib/gi18n.h>
@@ -60,8 +59,6 @@ do_colorshift (cairo_surface_t *dest, cairo_surface_t *src, int shift)
 	gint width, height, has_alpha, srcrowstride, destrowstride;
 	guchar *target_pixels;
 	guchar *original_pixels;
-	guchar *pixsrc;
-	guchar *pixdest;
 	int val;
 	guchar r,g,b;
 
@@ -74,8 +71,8 @@ do_colorshift (cairo_surface_t *dest, cairo_surface_t *src, int shift)
 	target_pixels = cairo_image_surface_get_data (dest);
 
 	for (i = 0; i < height; i++) {
-		pixdest = target_pixels + i*destrowstride;
-		pixsrc = original_pixels + i*srcrowstride;
+		guchar *pixdest = target_pixels + i*destrowstride;
+		guchar *pixsrc = original_pixels + i*srcrowstride;
 		for (j = 0; j < width; j++) {
 			r = *(pixsrc++);
 			g = *(pixsrc++);
@@ -180,7 +177,7 @@ button_widget_reload_surface (ButtonWidget *button)
 					 (button->priv->orientation & PANEL_HORIZONTAL_MASK) ? button->priv->size * scale: -1,
 					 &error);
 		if (error) {
-			//FIXME: this is not rendered at button->priv->size
+			/* FIXME: this is not rendered at button->priv->size */
 			GtkIconTheme *icon_theme = gtk_icon_theme_get_default();
 			button->priv->surface = gtk_icon_theme_load_surface (icon_theme,
 							       "image-missing",
@@ -317,7 +314,7 @@ calc_arrow (PanelOrientation  orientation,
 	switch (orientation) {
 	case PANEL_ORIENTATION_TOP:
 		*x     = (button_width - (*size)) / 2;
-		*y     = button_height * .99 - (*size) / (3/2);	// 3/2 is the approximate ratio of GTK arrows
+		*y     = button_height * .99 - (*size) / (3/2);	/* 3/2 is the approximate ratio of GTK arrows */
 		*angle = G_PI;
 		retval = GTK_ARROW_DOWN;
 		break;
@@ -328,7 +325,7 @@ calc_arrow (PanelOrientation  orientation,
 		retval = GTK_ARROW_UP;
 		break;
 	case PANEL_ORIENTATION_LEFT:
-		*x     = button_width * .99 - (*size) / (3/2);	// 3/2 is the approximate ratio of GTK arrows
+		*x     = button_width * .99 - (*size) / (3/2);	/* 3/2 is the approximate ratio of GTK arrows */
 		*y     = (button_height - (*size)) / 2;
 		*angle = G_PI / 2;
 		retval = GTK_ARROW_RIGHT;
