@@ -1,6 +1,7 @@
 /*
  * MATE panel drawer module.
  * (C) 1997 The Free Software Foundation
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * Authors: Miguel de Icaza
  *          Federico Mena
@@ -715,7 +716,7 @@ panel_drawer_set_dnd_enabled (Drawer   *drawer,
                              GDK_BUTTON1_MASK,
                              dnd_targets, 1,
                              GDK_ACTION_MOVE);
-        //FIXME: we're forgetting the use_custom_icon case, here
+        /* FIXME: we're forgetting the use_custom_icon case, here */
         gtk_drag_source_set_icon_name (drawer->button, button_widget_get_icon_name (BUTTON_WIDGET (drawer->button)));
 
         gtk_widget_set_has_window (drawer->button, FALSE);
@@ -727,12 +728,8 @@ panel_drawer_set_dnd_enabled (Drawer   *drawer,
 void
 drawer_query_deletion (Drawer *drawer)
 {
-    GtkWidget *dialog;
-
      if (drawer->toplevel) {
-        PanelWidget *panel_widget;
-
-        panel_widget = panel_toplevel_get_panel_widget (drawer->toplevel);
+        PanelWidget *panel_widget = panel_toplevel_get_panel_widget (drawer->toplevel);
 
         if (!panel_global_config_get_confirm_panel_remove () ||
             !g_list_length (panel_widget->applet_list)) {
@@ -740,7 +737,7 @@ drawer_query_deletion (Drawer *drawer)
                 return;
         }
 
-        dialog = panel_deletion_dialog (drawer->toplevel);
+        GtkWidget *dialog = panel_deletion_dialog (drawer->toplevel);
 
         g_signal_connect (dialog, "response", G_CALLBACK (drawer_deletion_response), drawer);
 
