@@ -3,6 +3,7 @@
  * Copyright (C) 2000 Helix Code, Inc.
  * Copyright (C) 2000 Eazel, Inc.
  * Copyright (C) 2004 Red Hat Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -61,10 +62,9 @@ static gboolean panel_menu_key_press_handler (GtkWidget   *widget,
 static inline gboolean desktop_is_home_dir(void)
 {
 	gboolean retval = FALSE;
-	GSettings *settings;
 
 	if (mate_gsettings_schema_exists (CAJA_PREFS_SCHEMA)) {
-		settings = g_settings_new (CAJA_PREFS_SCHEMA);
+		GSettings *settings = g_settings_new (CAJA_PREFS_SCHEMA);
 		retval = g_settings_get_boolean (settings, CAJA_PREFS_DESKTOP_IS_HOME_DIR_KEY);
 		g_object_unref (settings);
 	}
@@ -815,7 +815,7 @@ setup_uri_drag (GtkWidget  *menuitem,
 	g_signal_connect_data (G_OBJECT (menuitem), "drag_data_get",
 			       G_CALLBACK (drag_data_get_string_cb),
 			       g_strdup (uri),
-			       (GClosureNotify)g_free,
+			       (GClosureNotify) G_CALLBACK (g_free),
 			       0 /* connect_flags */);
 	g_signal_connect (G_OBJECT (menuitem), "drag_end",
 			  G_CALLBACK (drag_end_menu_cb), NULL);
@@ -846,7 +846,7 @@ setup_internal_applet_drag (GtkWidget             *menuitem,
 	g_signal_connect_data (G_OBJECT (menuitem), "drag_data_get",
 			       G_CALLBACK (drag_data_get_string_cb),
 			       g_strdup (panel_action_get_drag_id (type)),
-			       (GClosureNotify)g_free,
+			       (GClosureNotify) G_CALLBACK (g_free),
 			       0 /* connect_flags */);
 	g_signal_connect (G_OBJECT (menuitem), "drag_end",
 			  G_CALLBACK (drag_end_menu_cb), NULL);
@@ -855,31 +855,31 @@ setup_internal_applet_drag (GtkWidget             *menuitem,
 static void
 submenu_to_display (GtkWidget *menu)
 {
-	MateMenuTree           *tree;
-	MateMenuTreeDirectory  *directory;
-	const char          *menu_path;
-	void               (*append_callback) (GtkWidget *, gpointer);
-	gpointer             append_data;
+	void      (*append_callback) (GtkWidget *, gpointer);
+	gpointer  append_data;
 
 	if (!g_object_get_data (G_OBJECT (menu), "panel-menu-needs-loading"))
 		return;
 
 	g_object_set_data (G_OBJECT (menu), "panel-menu-needs-loading", NULL);
 
-	directory = g_object_get_data (G_OBJECT (menu),
-				       "panel-menu-tree-directory");
+	MateMenuTreeDirectory *directory =
+		g_object_get_data (G_OBJECT (menu),
+		                   "panel-menu-tree-directory");
 	if (!directory) {
-		menu_path = g_object_get_data (G_OBJECT (menu),
-					       "panel-menu-tree-path");
+		const char *menu_path =
+			g_object_get_data (G_OBJECT (menu),
+		                           "panel-menu-tree-path");
 		if (!menu_path)
 			return;
 
-		tree = g_object_get_data (G_OBJECT (menu), "panel-menu-tree");
+		MateMenuTree *tree =
+			g_object_get_data (G_OBJECT (menu),
+			                   "panel-menu-tree");
 		if (!tree)
 			return;
 
-		directory = matemenu_tree_get_directory_from_path (tree,
-								menu_path);
+		directory = matemenu_tree_get_directory_from_path (tree, menu_path);
 
 		g_object_set_data_full (G_OBJECT (menu),
 					"panel-menu-tree-directory",
@@ -1059,8 +1059,9 @@ create_menuitem (GtkWidget          *menu,
 				(GDestroyNotify) matemenu_tree_item_unref);
 
 	if (alias_directory) {
-		//FIXME: we should probably use this data when we do dnd or
-		//context menu for this menu item
+		/* FIXME: we should probably use this data when we do dnd or
+		 * context menu for this menu item
+		 */
 		g_object_set_data_full (G_OBJECT (menuitem),
 					"panel-menu-tree-alias-directory",
 					matemenu_tree_item_ref (alias_directory),
