@@ -2,6 +2,7 @@
  * panel-addto.c:
  *
  * Copyright (C) 2004 Vincent Untz
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -57,7 +58,7 @@
 #include "xstuff.h"
 #endif
 #ifdef HAVE_WAYLAND
-#include "gdk/gdkwayland.h"
+#include <gdk/gdkwayland.h>
 #endif
 
 typedef struct {
@@ -341,7 +342,7 @@ panel_addto_setup_drag (GtkTreeView          *tree_view,
 	g_signal_connect_data (G_OBJECT (tree_view), "drag_data_get",
 			       G_CALLBACK (panel_addto_drag_data_get_cb),
 			       g_strdup (text),
-			       (GClosureNotify) g_free,
+			       (GClosureNotify) G_CALLBACK (g_free),
 			       0 /* connect_flags */);
 	g_signal_connect_after (G_OBJECT (tree_view), "drag-begin",
 	                        G_CALLBACK (panel_addto_drag_begin_cb),
@@ -449,7 +450,6 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 			 GtkListStore *model,
 			 PanelAddtoItemInfo *applet)
 {
-	char *text;
 	GtkTreeIter iter;
 
 	if (applet == NULL) {
@@ -464,8 +464,8 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 	} else {
 		gtk_list_store_append (model, &iter);
 
-		text = panel_addto_make_text (applet->name,
-					      applet->description);
+		char *text = panel_addto_make_text (applet->name,
+		                                    applet->description);
 
 		gtk_list_store_set (model, &iter,
 				    COLUMN_ICON_NAME, applet->icon,
@@ -673,20 +673,18 @@ panel_addto_make_application_list (GSList             **parent_list,
 
 static void
 panel_addto_populate_application_model (GtkTreeStore *store,
-					GtkTreeIter  *parent,
-					GSList       *app_list)
+                                        GtkTreeIter  *parent,
+                                        GSList       *app_list)
 {
-	PanelAddtoAppList *data;
-	GtkTreeIter        iter;
-	char              *text;
-	GSList            *app;
+	GtkTreeIter  iter;
+	GSList      *app;
 
 	for (app = app_list; app != NULL; app = app->next) {
-		data = app->data;
-		gtk_tree_store_append (store, &iter, parent);
+		PanelAddtoAppList *data = app->data;
 
-		text = panel_addto_make_text (data->item_info.name,
-					      data->item_info.description);
+		gtk_tree_store_append (store, &iter, parent);
+		char *text = panel_addto_make_text (data->item_info.name,
+		                                    data->item_info.description);
 		gtk_tree_store_set (store, &iter,
 				    COLUMN_ICON_NAME, data->item_info.icon,
 				    COLUMN_TEXT, text,
@@ -831,7 +829,6 @@ panel_addto_dialog_response (GtkWidget *widget_dialog,
 {
 	GtkTreeSelection *selection;
 	GtkTreeModel     *filter_model;
-	GtkTreeModel     *child_model;
 	GtkTreeIter       iter;
 	GtkTreeIter       filter_iter;
 
@@ -850,7 +847,7 @@ panel_addto_dialog_response (GtkWidget *widget_dialog,
 			gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter_model),
 									  &iter,
 									  &filter_iter);
-			child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+			GtkTreeModel *child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
 			gtk_tree_model_get (child_model, &iter,
 					    COLUMN_DATA, &data, -1);
 
@@ -1185,7 +1182,6 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 	GtkTreeIter         iter;
 	GtkTreeIter         filter_iter;
 	PanelAddtoItemInfo *data;
-	char               *iid;
 
 	if (!gtk_tree_selection_get_selected (selection,
 					      &filter_model,
@@ -1230,7 +1226,9 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 		case PANEL_ADDTO_LAUNCHER_MENU:
 			gtk_tree_view_unset_rows_drag_source (GTK_TREE_VIEW (dialog->tree_view));
 			break;
-		case PANEL_ADDTO_MENU:
+		case PANEL_ADDTO_MENU: {
+			char *iid;
+
 			/* build the iid for menus other than the main menu */
 			if (data->iid == NULL) {
 				iid = g_strdup_printf ("MENU:%s/%s",
@@ -1243,6 +1241,7 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 							        iid);
 			g_free (iid);
 			break;
+		}
 		default:
 			panel_addto_setup_internal_applet_drag (GTK_TREE_VIEW (dialog->tree_view),
 							        data->iid);
@@ -1397,7 +1396,7 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 						     "sensitive", COLUMN_ENABLED,
 						     NULL);
 
-	//FIXME use the same search than the one for the search entry?
+	/* FIXME use the same search than the one for the search entry? */
 	gtk_tree_view_set_search_column (GTK_TREE_VIEW (dialog->tree_view),
 					 COLUMN_SEARCH);
 
