@@ -4,6 +4,7 @@
  *
  * Copyright (C) 2010 Carlos Garcia Campos <carlosgc@gnome.org>
  * Copyright (C) 2001 - 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -380,8 +381,6 @@ mate_panel_applet_frame_button_changed (GtkWidget      *widget,
 {
 	MatePanelAppletFrame *frame;
 	gboolean              handled = FALSE;
-	GdkDisplay *display;
-	GdkSeat *seat;
 
 	frame = MATE_PANEL_APPLET_FRAME (widget);
 
@@ -410,8 +409,8 @@ mate_panel_applet_frame_button_changed (GtkWidget      *widget,
 	case 3:
 		if (event->type == GDK_BUTTON_PRESS ||
 		    event->type == GDK_2BUTTON_PRESS) {
-			display = gtk_widget_get_display (widget);
-			seat = gdk_display_get_default_seat (display);
+			GdkDisplay *display = gtk_widget_get_display (widget);
+			GdkSeat    *seat    = gdk_display_get_default_seat (display);
 			gdk_seat_ungrab (seat);
 
 			MATE_PANEL_APPLET_FRAME_GET_CLASS (frame)->popup_menu (frame,
@@ -807,7 +806,7 @@ _mate_panel_applet_frame_applet_broken (MatePanelAppletFrame *frame)
 					      gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (dialog))));
 	else
 #endif
-	{ // Not using X11
+	{ /* Not using X11 */
 		gtk_window_present(GTK_WINDOW (dialog));
 	}
 
