@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010 Carlos Garcia Campos <carlosgc@gnome.org>
  * Copyright (C) 2010 Vincent Untz <vuntz@gnome.org>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,8 +50,7 @@ static GSList *mate_panel_applets_managers = NULL;
 static void
 _mate_panel_applets_manager_cleanup (gpointer data)
 {
-	g_slist_foreach (mate_panel_applets_managers, (GFunc) g_object_unref, NULL);
-	g_slist_free (mate_panel_applets_managers);
+	g_slist_free_full (mate_panel_applets_managers, g_object_unref);
 	mate_panel_applets_managers = NULL;
 }
 
