@@ -2,6 +2,7 @@
  * panel-background.h: panel background rendering
  *
  * Copyright (C) 2002, 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,10 +34,6 @@
 #include "panel-enums.h"
 #include "panel-types.h"
 
-#ifdef HAVE_X11
-#include "panel-background-monitor.h"
-#endif
-
 typedef struct _PanelBackground PanelBackground;
 
 typedef void (*PanelBackgroundChangedNotify)
@@ -59,12 +56,6 @@ struct _PanelBackground {
 	GdkPixbuf              *transformed_image;
 	cairo_pattern_t        *composited_pattern;
 
-#ifdef HAVE_X11
-	PanelBackgroundMonitor *monitor;
-	GdkPixbuf              *desktop;
-	gulong                  monitor_signal;
-#endif // HAVE_X11
-
 	GdkWindow              *window;
 	cairo_pattern_t        *default_pattern;
 	GdkRGBA                 default_color;
@@ -93,8 +84,6 @@ void  panel_background_set               (PanelBackground     *background,
 					  gboolean             rotate_image);
 void  panel_background_set_type          (PanelBackground     *background,
 					  PanelBackgroundType  type);
-void  panel_background_set_opacity       (PanelBackground     *background,
-					  guint16              opacity);
 void  panel_background_set_color         (PanelBackground     *background,
 					  const GdkRGBA             *color);
 void  panel_background_set_image         (PanelBackground     *background,
