@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2003 Sun Microsystems, Inc.
  * Copyright (C) 2004 Vincent Untz
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -98,7 +99,7 @@ static void panel_menu_bar_setup_tooltip(PanelMenuBar* menubar)
 	panel_util_set_tooltip_text(menubar->priv->places_item, _("Access documents, folders and network places"));
 	panel_util_set_tooltip_text(menubar->priv->desktop_item, _("Change desktop appearance and behavior, get help, or log out"));
 
-	//FIXME: this doesn't handle the right-click case. Sigh.
+	/* FIXME: this doesn't handle the right-click case. Sigh. */
 	/* Hide tooltip if a menu is activated */
 	g_signal_connect(menubar->priv->applications_item, "activate", G_CALLBACK (panel_menu_bar_hide_tooltip_and_focus), menubar);
 	g_signal_connect(menubar->priv->places_item, "activate", G_CALLBACK (panel_menu_bar_hide_tooltip_and_focus), menubar);
@@ -108,24 +109,27 @@ static void panel_menu_bar_setup_tooltip(PanelMenuBar* menubar)
 	g_signal_connect(GTK_MENU_SHELL (menubar), "deactivate", G_CALLBACK (panel_menu_bar_reinit_tooltip), menubar);
 }
 
-static void panel_menu_bar_update_visibility (GSettings* settings, gchar* key, PanelMenuBar* menubar)
+static void
+panel_menu_bar_update_visibility (GSettings    *settings,
+                                  gchar        *key,
+                                  PanelMenuBar *menubar)
 {
-	GtkWidget* image;
-	gchar *str;
-	GtkIconSize icon_size;
-	gint icon_height;
-
 	if (!GTK_IS_WIDGET (menubar))
 		return;
 
-	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->applications_item), g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_APPLICATIONS_KEY));
-	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->places_item), g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_PLACES_KEY));
-	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->desktop_item), g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_DESKTOP_KEY));
+	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->applications_item),
+	                        g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_APPLICATIONS_KEY));
+	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->places_item),
+	                        g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_PLACES_KEY));
+	gtk_widget_set_visible (GTK_WIDGET (menubar->priv->desktop_item),
+	                        g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_DESKTOP_KEY));
+
+	if (g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_ICON_KEY)) {
+		GtkWidget   *image;
+		gint         icon_height;
+		gchar       *str = g_settings_get_string (settings, PANEL_MENU_BAR_ICON_NAME_KEY);
+		GtkIconSize  icon_size = panel_menu_bar_icon_get_size ();
 
-	if (g_settings_get_boolean (settings, PANEL_MENU_BAR_SHOW_ICON_KEY))
-	{
-		str = g_settings_get_string (settings, PANEL_MENU_BAR_ICON_NAME_KEY);
-		icon_size = panel_menu_bar_icon_get_size ();
 		gtk_icon_size_lookup (icon_size, NULL, &icon_height);
 		if (str != NULL && str[0] != 0)
 			image = gtk_image_new_from_icon_name(str, icon_size);
