@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2005 Vincent Untz
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -249,7 +250,7 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 	g_signal_connect_data (item, "activate",
 			       G_CALLBACK (panel_menu_item_activate_desktop_file),
 			       g_strdup (full_path),
-			       (GClosureNotify) g_free, 0);
+			       (GClosureNotify) G_CALLBACK (g_free), 0);
 	g_signal_connect (G_OBJECT (item), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
 
@@ -297,7 +298,7 @@ panel_menu_items_append_place_item (const char *icon_name,
 
 	user_data = g_strdup (uri);
 	g_signal_connect_data (item, "activate", callback, user_data,
-			       (GClosureNotify) g_free, 0);
+			       (GClosureNotify) G_CALLBACK (g_free), 0);
 
 	g_signal_connect (G_OBJECT (item), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
@@ -451,8 +452,7 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu, guint max_items_or_
 	}
 
 	g_hash_table_destroy (table);
-	g_slist_foreach (lines, (GFunc) g_free, NULL);
-	g_slist_free (lines);
+	g_slist_free_full (lines, g_free);
 
 	add_bookmarks = g_slist_reverse (add_bookmarks);
 
@@ -520,7 +520,7 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu, guint max_items_or_
 
 		gicon = g_themed_icon_new_with_default_fallbacks (icon);
 
-		//FIXME: drag and drop will be broken for x-caja-search uris
+		/* FIXME: drag and drop will be broken for x-caja-search uris */
 		panel_menu_items_append_place_item (icon, gicon,
 						    label,
 						    tooltip,
@@ -569,8 +569,9 @@ drive_poll_for_media_cb (GObject      *source_object,
 		g_error_free (error);
 	}
 
-	//FIXME: should we mount the volume and activate the root of the new
-	//mount?
+	/* FIXME: should we mount the volume and activate the root of the new
+	 * mount?
+	 */
 }
 
 static void
@@ -612,7 +613,7 @@ panel_menu_item_append_drive (GtkWidget *menu,
 	g_signal_connect_data (item, "activate",
 			       G_CALLBACK (panel_menu_item_rescan_drive),
 			       g_object_ref (drive),
-			       (GClosureNotify) g_object_unref, 0);
+			       (GClosureNotify) G_CALLBACK (g_object_unref), 0);
 
 	g_signal_connect (G_OBJECT (item), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
@@ -712,7 +713,7 @@ panel_menu_item_append_volume (GtkWidget *menu,
 	g_signal_connect_data (item, "activate",
 			       G_CALLBACK (panel_menu_item_mount_volume),
 			       g_object_ref (volume),
-			       (GClosureNotify) g_object_unref, 0);
+			       (GClosureNotify) G_CALLBACK (g_object_unref), 0);
 
 	g_signal_connect (G_OBJECT (item), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
@@ -736,7 +737,7 @@ panel_menu_item_append_mount (GtkWidget *menu,
 
 	panel_menu_items_append_place_item (NULL, icon,
 					    display_name,
-					    display_name, //FIXME tooltip
+					    display_name, /* FIXME tooltip */
 					    menu,
 					    G_CALLBACK (activate_uri),
 					    activation_uri);
@@ -903,19 +904,19 @@ panel_place_menu_item_append_local_gio (PanelPlaceMenuItem *place_item,
 	if (g_slist_length (items) <= g_settings_get_uint (place_item->priv->menubar_settings, PANEL_MENU_BAR_MAX_ITEMS_OR_SUBMENU)) {
 		add_menu = menu;
 	} else {
-		GtkWidget  *item;
+		GtkWidget  *menu_item;
 
-		item = gtk_image_menu_item_new ();
-		setup_menuitem_with_icon (item, panel_menu_icon_get_size (),
-					  NULL,
-					  PANEL_ICON_REMOVABLE_MEDIA,
-					   _("Removable Media"));
+		menu_item = gtk_image_menu_item_new ();
+		setup_menuitem_with_icon (menu_item, panel_menu_icon_get_size (),
+		                          NULL,
+		                          PANEL_ICON_REMOVABLE_MEDIA,
+		                          _("Removable Media"));
 
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		gtk_widget_show (menu_item);
 
 		add_menu = create_empty_menu ();
-		gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), add_menu);
+		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), add_menu);
 	}
 
 	for (sl = items; sl; sl = sl->next) {
@@ -1025,6 +1026,7 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 	char      *name;
 	char      *uri;
 	GFile     *file;
+	int        recent_items_limit;
 
 	places_menu = panel_create_menu ();
 
@@ -1120,8 +1122,12 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 						      NULL,
 						      FALSE);
 
+	recent_items_limit = g_settings_get_int (place_item->priv->menubar_settings,
+						 PANEL_MENU_BAR_MAX_RECENT_ITEMS);
+
 	panel_recent_append_documents_menu (places_menu,
-					    place_item->priv->recent_manager);
+					    place_item->priv->recent_manager,
+					    recent_items_limit);
 /* Fix any failures of compiz/other wm's to communicate with gtk for transparency */
 	GtkWidget *toplevel = gtk_widget_get_toplevel (places_menu);
 	GdkScreen *screen = gtk_widget_get_screen(GTK_WIDGET(toplevel));
@@ -1380,6 +1386,10 @@ panel_place_menu_item_init (PanelPlaceMenuItem *menuitem)
 			"changed::" PANEL_MENU_BAR_MAX_ITEMS_OR_SUBMENU,
 			G_CALLBACK (panel_place_menu_item_key_changed),
 			G_OBJECT (menuitem));
+	g_signal_connect (menuitem->priv->menubar_settings,
+			"changed::" PANEL_MENU_BAR_MAX_RECENT_ITEMS,
+			G_CALLBACK (panel_place_menu_item_key_changed),
+			G_OBJECT (menuitem));
 
 	menuitem->priv->recent_manager = gtk_recent_manager_get_default ();
 
