@@ -2,6 +2,7 @@
  * panel-profile.c:
  *
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +24,6 @@
  */
 
 #include <config.h>
-#include <math.h>
 
 #include "panel-profile.h"
 #include "panel-layout.h"
@@ -238,20 +238,6 @@ panel_profile_get_background_type (PanelToplevel *toplevel)
 void
 panel_profile_set_background_color (PanelToplevel *toplevel,
 				    GdkRGBA       *color)
-{
-	panel_profile_set_background_gdk_rgba (toplevel, color);
-}
-
-void
-panel_profile_get_background_color (PanelToplevel *toplevel,
-				    GdkRGBA       *color)
-{
-	panel_profile_get_background_gdk_rgba (toplevel, color);
-}
-
-void
-panel_profile_set_background_gdk_rgba (PanelToplevel *toplevel,
-					GdkRGBA      *color)
 {
 	char *color_str;
 
@@ -263,8 +249,8 @@ panel_profile_set_background_gdk_rgba (PanelToplevel *toplevel,
 }
 
 void
-panel_profile_get_background_gdk_rgba (PanelToplevel *toplevel,
-					GdkRGBA      *color)
+panel_profile_get_background_color (PanelToplevel *toplevel,
+				    GdkRGBA       *color)
 {
 	char *color_str;
 
@@ -281,20 +267,20 @@ panel_profile_get_background_gdk_rgba (PanelToplevel *toplevel,
 
 void
 panel_profile_set_background_opacity (PanelToplevel *toplevel,
-				      guint16        opacity)
+                                      gdouble        percentage)
 {
 	GdkRGBA color;
 	panel_profile_get_background_color (toplevel, &color);
-	color.alpha = opacity / 65535.0;
+	color.alpha = percentage / 100.0;
 	panel_profile_set_background_color (toplevel, &color);
 }
 
-guint16
+gdouble
 panel_profile_get_background_opacity (PanelToplevel *toplevel)
 {
 	GdkRGBA color;
 	panel_profile_get_background_color (toplevel, &color);
-	return (guint16) round (color.alpha * 65535);
+	return color.alpha * 100.0;
 }
 
 void
@@ -493,22 +479,6 @@ panel_profile_is_writable_attached_tooltip (PanelToplevel *toplevel)
 	return is_writable;
 }
 
-static void
-get_background_color (PanelToplevel *toplevel,
-					  GdkRGBA       *color)
-{
-	char       *color_str;
-	color_str = g_settings_get_string (toplevel->background_settings, "color");
-	if (!color_str || !gdk_rgba_parse (color, color_str)) {
-		color->red   = 0;
-		color->green = 0;
-		color->blue  = 0;
-		color->alpha  = 1;
-	}
-
-	g_free (color_str);
-}
-
 static char *
 get_background_image (PanelToplevel *toplevel,
 		      gboolean     *fit,
@@ -540,7 +510,7 @@ panel_profile_load_background (PanelToplevel *toplevel)
 	background = &panel_widget->toplevel->background;
 	background_type = panel_profile_get_background_type (toplevel);
 
-	get_background_color (toplevel, &color);
+	panel_profile_get_background_color (toplevel, &color);
 
 	image = get_background_image (toplevel, &fit, &stretch, &rotate);
 
@@ -986,7 +956,7 @@ panel_profile_create_toplevel (GdkScreen *screen)
 	if (GDK_IS_X11_SCREEN (screen)) {
 		screen_number = gdk_x11_screen_get_screen_number (screen);
 	}
-#endif // HAVE_X11
+#endif /* HAVE_X11 */
 
 	g_settings_set_int (settings, PANEL_TOPLEVEL_SCREEN_KEY, screen_number);
 
