@@ -2,6 +2,7 @@
  * panel-profile.h:
  *
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -114,14 +115,9 @@ void        panel_profile_set_background_color        (PanelToplevel       *topl
 void        panel_profile_get_background_color        (PanelToplevel       *toplevel,
 						       GdkRGBA             *color);
 
-void        panel_profile_set_background_gdk_rgba    (PanelToplevel       *toplevel,
-						       GdkRGBA            *color);
-void        panel_profile_get_background_gdk_rgba    (PanelToplevel       *toplevel,
-						       GdkRGBA            *color);
-
 void        panel_profile_set_background_opacity      (PanelToplevel       *toplevel,
-						       guint16              opacity);
-guint16     panel_profile_get_background_opacity      (PanelToplevel       *toplevel);
+                                                       gdouble              percentage);
+gdouble     panel_profile_get_background_opacity      (PanelToplevel       *toplevel);
 
 void        panel_profile_set_background_image        (PanelToplevel       *toplevel,
 						       const char          *image);
