@@ -2,6 +2,7 @@
  * panel-properties-dialog.c:
  *
  * Copyright (C) 2003 Sun Microsystems, Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -238,7 +239,7 @@ panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->size_spin),
 				   panel_profile_get_toplevel_size (dialog->toplevel));
 
-	g_signal_connect_swapped (dialog->size_spin, "value_changed",
+	g_signal_connect_swapped (dialog->size_spin, "value-changed",
 				  G_CALLBACK (panel_properties_dialog_size_changed),
 				  dialog);
 
@@ -332,7 +333,7 @@ panel_properties_dialog_color_changed (PanelPropertiesDialog *dialog,
 	g_assert (dialog->color_button == GTK_WIDGET (color_button));
 
 	gtk_color_chooser_get_rgba (color_button, &color);
-	panel_profile_set_background_gdk_rgba (dialog->toplevel, &color);
+	panel_profile_set_background_color (dialog->toplevel, &color);
 	panel_properties_dialog_opacity_changed (dialog);
 }
 
@@ -349,8 +350,11 @@ panel_properties_dialog_setup_color_button (PanelPropertiesDialog *dialog,
 
 	panel_profile_get_background_color (dialog->toplevel, &color);
 
+	if (!gdk_screen_is_composited (gdk_screen_get_default ()))
+		color.alpha = 1.0;
+
 	gtk_color_chooser_set_rgba (GTK_COLOR_CHOOSER (dialog->color_button),
-				     &color);
+	                            &color);
 
 	g_signal_connect_swapped (dialog->color_button, "color_set",
 				  G_CALLBACK (panel_properties_dialog_color_changed),
@@ -416,7 +420,6 @@ static void
 panel_properties_dialog_opacity_changed (PanelPropertiesDialog *dialog)
 {
 	gdouble percentage;
-	guint16 opacity;
 
 	percentage = gtk_range_get_value (GTK_RANGE (dialog->opacity_scale));
 
@@ -425,16 +428,14 @@ panel_properties_dialog_opacity_changed (PanelPropertiesDialog *dialog)
 	else if (percentage <= 2)
 		percentage = 0;
 
-	opacity = (percentage / 100) * 65535;
-
-	panel_profile_set_background_opacity (dialog->toplevel, opacity);
+	panel_profile_set_background_opacity (dialog->toplevel, percentage);
 }
 
 static void
 panel_properties_dialog_setup_opacity_scale (PanelPropertiesDialog *dialog,
 					     GtkBuilder            *gui)
 {
-	guint16 opacity;
+	gboolean slider_active;
 	gdouble percentage;
 
 	dialog->opacity_scale = PANEL_GTK_BUILDER_GET (gui, "opacity_scale");
@@ -444,22 +445,30 @@ panel_properties_dialog_setup_opacity_scale (PanelPropertiesDialog *dialog,
 	dialog->opacity_legend = PANEL_GTK_BUILDER_GET (gui, "opacity_legend");
 	g_return_if_fail (dialog->opacity_legend != NULL);
 
-	opacity = panel_profile_get_background_opacity (dialog->toplevel);
+	slider_active = gdk_screen_is_composited (gdk_screen_get_default ());
 
-	percentage = (opacity * 100.0) / 65535;
+	if (slider_active) {
+		percentage = panel_profile_get_background_opacity (dialog->toplevel);
+	} else {
+		percentage = 100.0;
+	}
 
 	gtk_range_set_value (GTK_RANGE (dialog->opacity_scale), percentage);
 
-	g_signal_connect_swapped (dialog->opacity_scale, "value_changed",
-				  G_CALLBACK (panel_properties_dialog_opacity_changed),
-				  dialog);
+	if (!panel_profile_background_key_is_writable (dialog->toplevel, "opacity")) {
+		slider_active = FALSE;
+		gtk_widget_show (dialog->writability_warn_background);
+	}
 
-	if ( ! panel_profile_background_key_is_writable (dialog->toplevel, "opacity")) {
+	if (!slider_active) {
 		gtk_widget_set_sensitive (dialog->opacity_scale, FALSE);
 		gtk_widget_set_sensitive (dialog->opacity_label, FALSE);
 		gtk_widget_set_sensitive (dialog->opacity_legend, FALSE);
-		gtk_widget_show (dialog->writability_warn_background);
 	}
+
+	g_signal_connect_swapped (dialog->opacity_scale, "value-changed",
+				  G_CALLBACK (panel_properties_dialog_opacity_changed),
+				  dialog);
 }
 
 static void
@@ -603,7 +612,7 @@ panel_properties_dialog_update_orientation (PanelPropertiesDialog *dialog,
 	int                   profile_size;
 
 	/* change the maximum size of the panel */
-	//TODO: we should also do this when the monitor size changes
+	/* TODO: we should also do this when the monitor size changes */
 	max_size = panel_toplevel_get_maximum_size (dialog->toplevel);
 	spin_size = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (dialog->size_spin));
 	profile_size = panel_profile_get_toplevel_size (dialog->toplevel);
@@ -822,11 +831,11 @@ panel_properties_dialog_update_for_attached (PanelPropertiesDialog *dialog,
 }
 
 static PanelPropertiesDialog *
-panel_properties_dialog_new (PanelToplevel *toplevel,
-			     GtkBuilder    *gui)
+panel_properties_dialog_new (PanelToplevel *toplevel)
 {
 	PanelPropertiesDialog *dialog;
 	char                  *toplevel_settings_path;
+	GtkBuilder            *gui;
 
 	dialog = g_new0 (PanelPropertiesDialog, 1);
 
@@ -837,6 +846,9 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 
 	dialog->toplevel = toplevel;
 
+	gui = gtk_builder_new_from_resource (PANEL_RESOURCE_PATH "panel-properties-dialog.ui");
+	gtk_builder_set_translation_domain (gui, GETTEXT_PACKAGE);
+
 	dialog->properties_dialog = PANEL_GTK_BUILDER_GET (gui, "panel_properties_dialog");
 	g_signal_connect_swapped (dialog->properties_dialog, "response",
 				  G_CALLBACK (panel_properties_dialog_response), dialog);
@@ -900,9 +912,7 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	panel_toplevel_push_autohide_disabler (dialog->toplevel);
 	panel_widget_register_open_dialog (panel_toplevel_get_panel_widget (dialog->toplevel),
 					   dialog->properties_dialog);
-
-	gtk_widget_show (dialog->properties_dialog);
-
+	g_object_unref (gui);
 	return dialog;
 }
 
@@ -910,7 +920,6 @@ void
 panel_properties_dialog_present (PanelToplevel *toplevel)
 {
 	PanelPropertiesDialog *dialog;
-	GtkBuilder            *gui;
 
 	if (!panel_properties_dialog_quark)
 		panel_properties_dialog_quark =
@@ -921,16 +930,8 @@ panel_properties_dialog_present (PanelToplevel *toplevel)
 		gtk_window_set_screen (GTK_WINDOW (dialog->properties_dialog),
 				       gtk_window_get_screen (GTK_WINDOW (toplevel)));
 		gtk_window_present (GTK_WINDOW (dialog->properties_dialog));
-		return;
+	} else {
+		dialog = panel_properties_dialog_new (toplevel);
+		gtk_widget_show (dialog->properties_dialog);
 	}
-
-	gui = gtk_builder_new ();
-	gtk_builder_set_translation_domain (gui, GETTEXT_PACKAGE);
-	gtk_builder_add_from_resource (gui,
-	                               PANEL_RESOURCE_PATH "panel-properties-dialog.ui",
-	                               NULL);
-
-	dialog = panel_properties_dialog_new (toplevel, gui);
-
-	g_object_unref (gui);
 }
