@@ -3,6 +3,7 @@
  * panel-recent.c
  *
  * Copyright (C) 2002 James Willcox <jwillcox@gnome.org>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -65,13 +66,13 @@ recent_documents_activate_cb (GtkRecentChooser *chooser,
 	recent_info = gtk_recent_chooser_get_current_item (chooser);
 	uri = gtk_recent_info_get_uri (recent_info);
 	mime_type = gtk_recent_info_get_mime_type (recent_info);
-	//FIXME gtk_recent_info_get_application_info() could be useful
+	/* FIXME gtk_recent_info_get_application_info() could be useful */
 
 	if (show_uri (uri, mime_type, screen, &error) != TRUE) {
 		char *uri_utf8;
 
 		uri_utf8 = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
-		//FIXME this could fail... Maybe we want gtk_recent_info_get_display_name()
+		/* FIXME this could fail... Maybe we want gtk_recent_info_get_display_name() */
 
 		if (error) {
 			char *primary;
@@ -185,7 +186,8 @@ recent_documents_clear_cb (GtkMenuItem      *menuitem,
 
 void
 panel_recent_append_documents_menu (GtkWidget        *top_menu,
-				    GtkRecentManager *manager)
+				    GtkRecentManager *manager,
+				    int               recent_items_limit)
 {
 	GtkWidget      *recent_menu;
 	GtkWidget      *menu_item;
@@ -199,6 +201,9 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 				  _("Recent Documents"));
 	recent_menu = gtk_recent_chooser_menu_new_for_manager (manager);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), recent_menu);
+	
+	gtk_recent_chooser_set_limit (GTK_RECENT_CHOOSER (recent_menu),
+				      recent_items_limit);
 
 	g_signal_connect (G_OBJECT (recent_menu), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
@@ -218,7 +223,7 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 			  G_CALLBACK (recent_documents_activate_cb),
 			  NULL);
 
-	//FIXME this is not possible with GtkRecent...: egg_recent_view_gtk_set_icon_size (view, panel_menu_icon_get_size ());
+	/* FIXME this is not possible with GtkRecent...: egg_recent_view_gtk_set_icon_size (view, panel_menu_icon_get_size ()); */
 
 	g_signal_connect_object (manager, "changed",
 				 G_CALLBACK (panel_recent_manager_changed_cb),
