@@ -2,6 +2,7 @@
  * panel-reset.c
  *
  * Copyright (C) 2010 Perberos <perberos@gmail.com>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,7 +28,7 @@
 #define __PANEL_RESET_C__
 
 #include <stdlib.h>
-#include <glib.h>
+#include <gio/gio.h>
 #include "panel-reset.h"
 #include "panel-schemas.h"
 
