@@ -2,6 +2,7 @@
  * panel-run-dialog.c:
  *
  * Copyright (C) 2003 Frank Worsley <fworsley@shaw.ca>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -88,7 +89,7 @@ typedef struct {
 	GHashTable       *dir_hash;
 	GList		 *possible_executables;
 	GList		 *completion_items;
-	GCompletion      *completion;
+	GtkEntryCompletion *completion;
 
 	int	          add_items_idle_id;
 	int		  find_command_idle_id;
@@ -158,22 +159,19 @@ _panel_run_save_recent_programs_list (PanelRunDialog   *dialog,
 				      GtkComboBox      *entry,
 				      char             *lastcommand)
 {
-	GtkTreeModel *model;
-	GtkTreeIter   iter;
-	guint         history_max_size;
-	gboolean      history_reverse;
-
-	history_reverse = g_settings_get_boolean (dialog->settings, PANEL_RUN_HISTORY_REVERSE_KEY);
-	history_max_size = g_settings_get_uint (dialog->settings, PANEL_RUN_HISTORY_MAX_SIZE_KEY);
+	gboolean history_reverse =
+		g_settings_get_boolean (dialog->settings, PANEL_RUN_HISTORY_REVERSE_KEY);
+	guint history_max_size =
+		g_settings_get_uint (dialog->settings, PANEL_RUN_HISTORY_MAX_SIZE_KEY);
 
 	if (history_max_size == 0)
 		g_settings_set_strv (dialog->settings, PANEL_RUN_HISTORY_KEY, NULL);
 	else {
-		model = gtk_combo_box_get_model (GTK_COMBO_BOX (entry));
-
+		GtkTreeIter   iter;
+		GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (entry));
 		/* reasonable upper bound for zero-terminated array with new command */
-		gchar *items[gtk_tree_model_iter_n_children (model, NULL) + 2];
-		guint  items_added = 0;
+		gchar        *items[gtk_tree_model_iter_n_children (model, NULL) + 2];
+		guint         items_added = 0;
 
 		items[0] = lastcommand;
 
@@ -255,10 +253,6 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 	g_list_free (dialog->completion_items);
 	dialog->completion_items = NULL;
 
-	if (dialog->completion)
-		g_completion_free (dialog->completion);
-	dialog->completion = NULL;
-
 	panel_run_dialog_disconnect_pixmap (dialog);
 
 	g_free (dialog);
@@ -320,8 +314,9 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 		 */
 		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->pixmap), pixbuf);
 
-		//FIXME: it'd be better to set an icon of the correct size,
-		//(ditto for the drag icon?)
+		/* FIXME: it'd be better to set an icon of the correct size,
+		 * (ditto for the drag icon?)
+		 */
 		gtk_window_set_icon (GTK_WINDOW (dialog->run_dialog), pixbuf);
 
 		gtk_drag_source_set_icon_gicon (dialog->run_dialog, dialog->icon);
@@ -337,6 +332,9 @@ command_is_executable (const char   *command,
 		       char       ***argvp)
 {
 	gboolean   result;
+	GRegex    *regex = NULL;
+	gsize      argv_len;
+	g_autofree gchar *home_path = NULL;
 	char     **argv;
 	char      *path;
 	int        argc;
@@ -346,6 +344,18 @@ command_is_executable (const char   *command,
 	if (!result)
 		return FALSE;
 
+	regex = g_regex_new ("^~/", 0, 0, NULL);
+	argv_len = g_strv_length (argv);
+	home_path = g_build_filename (g_get_home_dir (), "/", NULL);
+	for (gsize i = 0; i < argv_len; i++)
+	{
+		gchar *tmp_argv = NULL;
+
+		tmp_argv = g_regex_replace_literal (regex, argv[i], -1, 0, home_path, 0, NULL);
+		g_free (argv[i]);
+		argv[i] = tmp_argv;
+	}
+	g_regex_unref (regex);
 	path = g_find_program_in_path (argv[0]);
 
 	if (!path) {
@@ -571,7 +581,7 @@ panel_run_dialog_append_file_utf8 (PanelRunDialog *dialog,
 				   const char     *file)
 {
 	const char *text;
-	char       *quoted, *temp;
+	char       *quoted;
 	GtkWidget  *entry;
 
 	/* Don't allow filenames beginning with '-' */
@@ -582,7 +592,7 @@ panel_run_dialog_append_file_utf8 (PanelRunDialog *dialog,
 	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 	if (text && text [0]) {
-		temp = g_strconcat (text, " ", quoted, NULL);
+		char *temp = g_strconcat (text, " ", quoted, NULL);
 		gtk_entry_set_text (GTK_ENTRY (entry), temp);
 		g_free (temp);
 	} else
@@ -755,7 +765,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	gtk_tree_path_free (path);
 
 	panel_run_dialog_set_icon (dialog, found_icon, FALSE);
-	//FIXME update dialog->program_label
+	/* FIXME update dialog->program_label */
 
 	g_clear_object (&found_icon);
 	g_free (text);
@@ -945,7 +955,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->program_list),
 				 model_filter);
-	//FIXME use the same search than the fuzzy one?
+	/* FIXME use the same search than the fuzzy one? */
 	gtk_tree_view_set_search_column (GTK_TREE_VIEW (dialog->program_list),
 					 COLUMN_NAME);
 
@@ -1017,7 +1027,7 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	GtkTreeIter   iter;
 	GtkTreeIter   filter_iter;
 	char         *temp;
-	char         *path, *stripped;
+	char         *path;
 	gboolean      terminal;
 	GKeyFile     *key_file;
 	GtkWidget    *entry;
@@ -1063,7 +1073,7 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
 	temp = panel_key_file_get_string (key_file, "Exec");
 	if (temp) {
-		stripped = remove_parameters (temp);
+		char *stripped = remove_parameters (temp);
 		gtk_entry_set_text (GTK_ENTRY (entry), stripped);
 		g_free (stripped);
 	} else {
@@ -1080,7 +1090,7 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	g_free (temp);
 
 	temp = panel_key_file_get_locale_string (key_file, "Comment");
-	//FIXME: if sure_string () == "", we should display "Will run..." as in entry_changed()
+	/* FIXME: if sure_string () == "", we should display "Will run..." as in entry_changed() */
 	gtk_label_set_text (GTK_LABEL (dialog->program_label),
 			    sure_string (temp));
 	g_free (temp);
@@ -1115,8 +1125,6 @@ static void
 panel_run_dialog_setup_program_list (PanelRunDialog *dialog,
 				     GtkBuilder     *gui)
 {
-	GtkTreeSelection *selection;
-
 	dialog->program_list = PANEL_GTK_BUILDER_GET (gui, "program_list");
 	dialog->program_list_box = PANEL_GTK_BUILDER_GET (gui, "program_list_box");
 	dialog->program_label = PANEL_GTK_BUILDER_GET (gui, "program_label");
@@ -1130,7 +1138,9 @@ panel_run_dialog_setup_program_list (PanelRunDialog *dialog,
 	g_object_ref (dialog->program_list_box);
 
 	if (panel_profile_get_enable_program_list ()) {
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->program_list));
+		GtkTreeSelection *selection =
+			gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->program_list));
+
 		gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
 	        g_signal_connect (selection, "changed",
@@ -1247,10 +1257,9 @@ file_button_browse_response (GtkWidget      *chooser,
 			     gint            response,
 			     PanelRunDialog *dialog)
 {
-	char *file;
 
 	if (response == GTK_RESPONSE_OK) {
-		file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+		char *file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 		panel_run_dialog_append_file (dialog, file);
 		g_free (file);
 	}
@@ -1366,7 +1375,6 @@ fill_possible_executables (void)
 
 	for (i = 0; pathv [i]; i++) {
 		const char *file;
-		char       *filename;
 		GDir       *dir;
 
 		dir = g_dir_open (pathv [i], 0, NULL);
@@ -1375,8 +1383,7 @@ fill_possible_executables (void)
 			continue;
 
 		while ((file = g_dir_read_name (dir))) {
-			filename = g_build_filename (pathv [i], file, NULL);
-			list = g_list_prepend (list, filename);
+			list = g_list_prepend (list, g_build_filename (pathv [i], file, NULL));
 		}
 
 		g_dir_close (dir);
@@ -1423,6 +1430,33 @@ fill_executables (GList *possible_executables,
 	return list;
 }
 
+static GtkTreeModel *
+create_completion_model (GList *list)
+{
+	GtkListStore *store;
+	GtkTreeIter   iter;
+	GList        *l;
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	for (l = list; l; l = l->next)
+	{
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, l->data, -1);
+	}
+
+	return GTK_TREE_MODEL (store);
+}
+
+static void
+completion_add_items (GtkEntryCompletion *completion, GList *list)
+{
+	GtkTreeModel          *completion_model;
+
+	completion_model = create_completion_model (list);
+	gtk_entry_completion_set_model (completion, completion_model);
+	g_object_unref (completion_model);
+}
+
 static void
 panel_run_dialog_update_completion (PanelRunDialog *dialog,
 				    const char     *text)
@@ -1440,14 +1474,6 @@ panel_run_dialog_update_completion (PanelRunDialog *dialog,
 	list = NULL;
 	executables = NULL;
 
-	if (!dialog->completion) {
-		dialog->completion = g_completion_new (NULL);
-		dialog->possible_executables = fill_possible_executables ();
-		dialog->dir_hash = g_hash_table_new_full (g_str_hash,
-							  g_str_equal,
-							  g_free, NULL);
-	}
-
 	buf = g_path_get_basename (text);
 	prefix = buf[0];
 	g_free (buf);
@@ -1491,8 +1517,7 @@ panel_run_dialog_update_completion (PanelRunDialog *dialog,
 	if (list == NULL)
 		return;
 
-	g_completion_add_items (dialog->completion, list);
-
+	completion_add_items (dialog->completion, list);
 	dialog->completion_items = g_list_concat (dialog->completion_items,
 						  list);
 }
@@ -1502,10 +1527,8 @@ entry_event (GtkEditable    *entry,
 	     GdkEventKey    *event,
 	     PanelRunDialog *dialog)
 {
-	GtkTreeSelection *selection;
 	char             *prefix;
 	char             *nospace_prefix;
-	char             *nprefix;
 	char             *temp;
 	int               pos, tmp;
 
@@ -1517,7 +1540,7 @@ entry_event (GtkEditable    *entry,
 	*/
 	dialog->use_program_list = FALSE;
 	if (panel_profile_get_enable_program_list ()) {
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->program_list));
+		GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->program_list));
 		gtk_tree_selection_unselect_all (selection);
 	}
 
@@ -1570,43 +1593,6 @@ entry_event (GtkEditable    *entry,
 			return FALSE;
 		}
 
-		pos = strlen (prefix);
-		nprefix = NULL;
-
-		g_completion_complete_utf8 (dialog->completion, nospace_prefix,
-					    &nprefix);
-
-		if (nprefix) {
-			int insertpos;
-			insertpos = 0;
-
-			temp = g_strndup (prefix, nospace_prefix - prefix);
-			g_free (prefix);
-
-			prefix = g_strconcat (temp, nprefix, NULL);
-
-			g_signal_handler_block (dialog->combobox,
-						dialog->changed_id);
-			gtk_editable_delete_text (entry, 0, -1);
-			g_signal_handler_unblock (dialog->combobox,
-						  dialog->changed_id);
-
-			gtk_editable_insert_text (entry,
-						  prefix, strlen (prefix),
-						  &insertpos);
-
- 			gtk_editable_set_position (entry, pos);
-			gtk_editable_select_region (entry, pos, -1);
-
-			dialog->completion_started = TRUE;
-
-			g_free (temp);
-			g_free (nprefix);
-			g_free (prefix);
-
-			return TRUE;
-		}
-
 		g_free (prefix);
 	}
 
@@ -1617,13 +1603,9 @@ static void
 combobox_changed (GtkComboBox    *combobox,
 		  PanelRunDialog *dialog)
 {
-	char *text;
-	char *start;
-	char *msg;
+	char *text = g_strdup (panel_run_dialog_get_combo_text (dialog));
+	char *start = text;
 
-	text = g_strdup (panel_run_dialog_get_combo_text (dialog));
-
-	start = text;
 	while (*start != '\0' && g_ascii_isspace (*start))
 		start++;
 
@@ -1686,8 +1668,8 @@ combobox_changed (GtkComboBox    *combobox,
 
 	if (panel_profile_get_enable_program_list () &&
 	    !dialog->use_program_list) {
-		msg = g_strdup_printf (_("Will run command: '%s'"),
-				       start);
+		char *msg = g_strdup_printf (_("Will run command: '%s'"),
+		                             start);
 		gtk_label_set_text (GTK_LABEL (dialog->program_label), msg);
 		g_free (msg);
 	}
@@ -1758,10 +1740,18 @@ panel_run_dialog_setup_entry (PanelRunDialog *dialog,
 	GtkWidget             *entry;
 
 	dialog->combobox = PANEL_GTK_BUILDER_GET (gui, "comboboxentry");
+	dialog->possible_executables = fill_possible_executables ();
+	dialog->dir_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
 	entry = gtk_bin_get_child (GTK_BIN (dialog->combobox));
 	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
 
+	dialog->completion = gtk_entry_completion_new ();
+	gtk_entry_completion_set_inline_completion (dialog->completion, TRUE);
+	gtk_entry_completion_set_popup_completion (dialog->completion, FALSE);
+	gtk_entry_set_completion (GTK_ENTRY (entry), dialog->completion);
+	gtk_entry_completion_set_text_column (dialog->completion, 0);
+
 	gtk_combo_box_set_model (GTK_COMBO_BOX (dialog->combobox),
 				 _panel_run_get_recent_programs_list (dialog));
 	gtk_combo_box_set_entry_text_column
