@@ -2,6 +2,7 @@
  * panel-run-dialog.h:
  *
  * Copyright (C) 2003 Frank Worsley <fworsley@shaw.ca>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,7 +26,7 @@
 #ifndef __PANEL_RUN_DIALOG_H__
 #define __PANEL_RUN_DIALOG_H__
 
-#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 
 G_BEGIN_DECLS
 
