@@ -6,6 +6,7 @@
  * Copyright 2000,2001 Eazel, Inc.
  * Copyright 2001 George Lebl
  * Copyright 2002 Sun Microsystems Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * Authors: George Lebl
  *          Jacob Berkman
@@ -28,7 +29,7 @@
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gdk/gdkx.h>
+#include <gdk/gdk.h>
 
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #include <libmate-desktop/mate-desktop-utils.h>
@@ -828,20 +829,15 @@ panel_util_get_icon_name_from_g_icon (GIcon *gicon)
 static char *
 panel_util_get_file_display_name_if_mount (GFile *file)
 {
-	GFile          *compare;
-	GVolumeMonitor *monitor;
-	GList          *mounts, *l;
-	char           *ret;
-
-	ret = NULL;
+	GList *l;
+	char  *ret = NULL;
 
 	/* compare with all mounts */
-	monitor = g_volume_monitor_get ();
-	mounts = g_volume_monitor_get_mounts (monitor);
+	GVolumeMonitor *monitor = g_volume_monitor_get ();
+	GList *mounts = g_volume_monitor_get_mounts (monitor);
 	for (l = mounts; l != NULL; l = l->next) {
-		GMount *mount;
-		mount = G_MOUNT (l->data);
-		compare = g_mount_get_root (mount);
+		GMount *mount = G_MOUNT (l->data);
+		GFile *compare = g_mount_get_root (mount);
 		if (!ret && g_file_equal (file, compare))
 			ret = g_mount_get_name (mount);
 		g_object_unref (compare);
@@ -860,13 +856,12 @@ panel_util_get_file_display_for_common_files (GFile *file)
 
 	compare = g_file_new_for_path (g_get_home_dir ());
 	if (g_file_equal (file, compare)) {
-		GSettings *caja_desktop_settings;
 		char *caja_home_icon_name = NULL;
 
 		g_object_unref (compare);
 
 		if (mate_gsettings_schema_exists (CAJA_DESKTOP_SCHEMA)) {
-			caja_desktop_settings = g_settings_new (CAJA_DESKTOP_SCHEMA);
+			GSettings *caja_desktop_settings = g_settings_new (CAJA_DESKTOP_SCHEMA);
 			caja_home_icon_name = g_settings_get_string (caja_desktop_settings,
 														 CAJA_DESKTOP_HOME_ICON_NAME_KEY);
 			g_object_unref (caja_desktop_settings);
@@ -915,17 +910,12 @@ panel_util_get_file_description (GFile *file)
 
 static char *
 panel_util_get_file_display_name (GFile    *file,
-				  gboolean  use_fallback)
+                                 gboolean  use_fallback)
 {
-	GFileInfo *info;
-	char      *ret;
-
-	ret = NULL;
-
-	info = g_file_query_info (file, "standard::display-name",
-				  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-				  NULL, NULL);
-
+	char *ret = NULL;
+	GFileInfo *info = g_file_query_info (file, "standard::display-name",
+	                                     G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+	                                     NULL, NULL);
 	if (info) {
 		ret = g_strdup (g_file_info_get_display_name (info));
 		g_object_unref (info);
@@ -933,9 +923,7 @@ panel_util_get_file_display_name (GFile    *file,
 
 	if (!ret && use_fallback) {
 		/* can happen with URI schemes non supported by gvfs */
-		char *basename;
-
-		basename = g_file_get_basename (file);
+		char *basename = g_file_get_basename (file);
 		ret = g_filename_display_name (basename);
 		g_free (basename);
 	}
@@ -946,20 +934,16 @@ panel_util_get_file_display_name (GFile    *file,
 static char *
 panel_util_get_file_icon_name_if_mount (GFile *file)
 {
-	GFile          *compare;
 	GVolumeMonitor *monitor;
 	GList          *mounts, *l;
-	char           *ret;
-
-	ret = NULL;
+	char           *ret = NULL;
 
 	/* compare with all mounts */
 	monitor = g_volume_monitor_get ();
 	mounts = g_volume_monitor_get_mounts (monitor);
 	for (l = mounts; l != NULL; l = l->next) {
-		GMount *mount;
-		mount = G_MOUNT (l->data);
-		compare = g_mount_get_root (mount);
+		GMount *mount = G_MOUNT (l->data);
+		GFile *compare = g_mount_get_root (mount);
 		if (!ret && g_file_equal (file, compare)) {
 			GIcon *gicon;
 			gicon = g_mount_get_icon (mount);
@@ -1057,7 +1041,7 @@ panel_util_get_label_for_uri (const char *text_uri)
 
 	label = NULL;
 
-	//FIXME: see caja_query_to_readable_string() to have a nice name
+	/* FIXME: see caja_query_to_readable_string() to have a nice name */
 	if (g_str_has_prefix (text_uri, "x-caja-search:"))
 		return g_strdup (_("Search"));
 
@@ -1214,7 +1198,7 @@ panel_util_set_tooltip_text (GtkWidget  *widget,
 	g_object_set (widget, "has-tooltip", TRUE, NULL);
 	g_signal_connect_data (widget, "query-tooltip",
 			       G_CALLBACK (panel_util_query_tooltip_cb),
-			       g_strdup (text), (GClosureNotify) g_free, 0);
+			       g_strdup (text), (GClosureNotify) G_CALLBACK (g_free), 0);
 }
 
 /* This is similar to what g_file_new_for_commandline_arg() does, but
