@@ -1,6 +1,7 @@
 /* Mate panel: Initialization routines
  * (C) 1997,1998,1999,2000 the Free Software Foundation
  * (C) 2000 Eazel, Inc.
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * Authors: Federico Mena
  *          Miguel de Icaza
@@ -19,7 +20,10 @@
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+
+#ifdef HAVE_X11
 #include <gtk/gtkx.h> /* for GTK_IS_SOCKET */
+#endif
 
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-gtk.h>
@@ -38,7 +42,6 @@
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
 #include "panel-separator.h"
-#include "panel-multiscreen.h"
 #include "panel-toplevel.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
@@ -109,7 +112,7 @@ orient_change_foreach(GtkWidget *w, gpointer data)
 {
 	AppletInfo *info = g_object_get_data (G_OBJECT (w), "applet_info");
 	PanelWidget *panel = data;
-	
+
 	orientation_change(info,panel);
 }
 
@@ -139,7 +142,7 @@ size_change_foreach(GtkWidget *w, gpointer data)
 {
 	AppletInfo *info = g_object_get_data (G_OBJECT (w), "applet_info");
 	PanelWidget *panel = data;
-	
+
 	size_change(info,panel);
 }
 
@@ -364,7 +367,9 @@ panel_popup_menu (PanelToplevel *toplevel,
 	gtk_menu_set_screen (GTK_MENU (menu),
 			     gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button, activate_time);
+	gtk_window_set_attached_to (GTK_WINDOW (gtk_widget_get_toplevel (menu)),
+				    GTK_WIDGET (toplevel));
+	gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
 
 	return TRUE;
 }
@@ -389,33 +394,56 @@ static gboolean
 panel_key_press_event (GtkWidget   *widget,
 		       GdkEventKey *event)
 {
+#ifdef HAVE_X11
 	/*
   	 * If the focus widget is a GtkSocket, i.e. the
-	 * focus is in an applet in another process, then key 
+	 * focus is in an applet in another process, then key
 	 * bindings do not work. We get around this by
 	 * activating the key bindings here.
-	 */ 
+	 *
+	 * Will always be false when not using X
+	 */
 	if (GTK_IS_SOCKET (gtk_window_get_focus (GTK_WINDOW (widget))) &&
 	    event->keyval == GDK_KEY_F10 &&
 	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
 		return gtk_bindings_activate (G_OBJECT (widget),
 					      event->keyval,
 					      event->state);
+#endif
 
 	return FALSE;
 }
 
+static gboolean
+can_set_background_image (PanelToplevel *toplevel,
+                          const gchar   *uri,
+                          gchar        **filename)
+{
+	gchar *image;
+
+	if (! panel_profile_background_key_is_writable (toplevel, "type") ||
+	    ! panel_profile_background_key_is_writable (toplevel, "image"))
+		return FALSE;
+
+	image = g_filename_from_uri (uri, NULL, NULL);
+	if (! image)
+		return FALSE;
+
+	if (filename)
+		*filename = image;
+	else
+		g_free (image);
+
+	return TRUE;
+}
+
 static gboolean
 set_background_image_from_uri (PanelToplevel *toplevel,
 			       const char    *uri)
 {
 	char *image;
 
-	if ( ! panel_profile_background_key_is_writable (toplevel, "type") ||
-	     ! panel_profile_background_key_is_writable (toplevel, "image"))
-		return FALSE;
-
-	if (!(image = g_filename_from_uri (uri, NULL, NULL)))
+	if (! can_set_background_image (toplevel, uri, &image))
 		return FALSE;
 
 	panel_profile_set_background_image (toplevel, image);
@@ -474,7 +502,7 @@ drop_url (PanelWidget *panel,
 		g_strfreev (netscape_url);
 		return FALSE;
 	}
-	
+
 	comment = g_strdup_printf (_("Open URL: %s"),
 				   netscape_url[NETSCAPE_URL_URL]);
 
@@ -590,16 +618,138 @@ drop_caja_desktop_uri (PanelWidget *panel,
 	return success;
 }
 
+enum {
+	RESPONSE_SET_BACKGROUND,
+	RESPONSE_CREATE_LAUNCHER
+};
+
+static void
+image_drop_dialog_response (GtkWidget   *dialog,
+                            int          response,
+                            PanelWidget *panel)
+{
+	if (response == RESPONSE_SET_BACKGROUND) {
+		const gchar *uri = g_object_get_data (G_OBJECT (dialog), "uri");
+
+		set_background_image_from_uri (panel->toplevel, uri);
+	} else if (response == RESPONSE_CREATE_LAUNCHER) {
+		const gchar *uri = g_object_get_data (G_OBJECT (dialog), "uri");
+		gint pos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "pos"));
+
+		drop_uri (panel, pos, uri, PANEL_ICON_UNKNOWN);
+	}
+
+	gtk_widget_destroy (dialog);
+}
+
+static GtkWidget *
+image_drop_dialog (PanelToplevel *toplevel,
+                   const gchar   *uri)
+{
+	GtkWidget *dialog;
+	GFile *file;
+	GFileInfo *info;
+	gchar *display_name;
+	gchar *secondary_p1;
+	const gchar *secondary_p2;
+
+	file = g_file_new_for_uri (uri);
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
+	                          G_FILE_QUERY_INFO_NONE, NULL, NULL);
+	if (info) {
+		display_name = g_strdup (g_file_info_get_display_name (info));
+		g_object_unref (info);
+	} else {
+		display_name = g_file_get_basename (file);
+	}
+	g_object_unref (file);
+
+	dialog = gtk_message_dialog_new (
+			GTK_WINDOW (toplevel),
+			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_MESSAGE_QUESTION,
+			GTK_BUTTONS_NONE,
+			_("You dropped an image on the panel, what do you want to do?"));
+
+	secondary_p1 = g_strdup_printf (_("You dropped the image \"%s\" on the panel."),
+	                                display_name);
+	if (panel_profile_id_lists_are_writable ())
+		secondary_p2 = _("Do you want to create a launcher for it or "
+		                 "use it as the panel's background?");
+	else
+		secondary_p2 = _("Do you want to use it as the panel's background?");
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	                                          "%s %s",
+	                                          secondary_p1, secondary_p2);
+
+	g_free (secondary_p1);
+	g_free (display_name);
+
+	panel_dialog_add_button (GTK_DIALOG (dialog),
+	                         _("_Cancel"), "process-stop",
+	                         GTK_RESPONSE_CANCEL);
+	if (panel_profile_id_lists_are_writable ()) {
+		panel_dialog_add_button (GTK_DIALOG (dialog),
+		                         _("Create _Launcher"), "mate-panel-launcher",
+		                         RESPONSE_CREATE_LAUNCHER);
+	}
+	panel_dialog_add_button (GTK_DIALOG (dialog),
+	                         _("Set _Background Image"), "background",
+	                         RESPONSE_SET_BACKGROUND);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+	                       gtk_window_get_screen (GTK_WINDOW (toplevel)));
+
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+
+	g_signal_connect_swapped (dialog, "destroy",
+	                          G_CALLBACK (panel_toplevel_pop_autohide_disabler),
+	                          toplevel);
+
+	panel_toplevel_push_autohide_disabler (toplevel);
+
+	return dialog;
+}
+
+static gboolean
+ask_about_image_drop (PanelWidget *panel,
+                      int          pos,
+                      const char  *uri)
+{
+	GtkWidget *dialog;
+
+	/* if it's not possible to set the background image, add a launcher */
+	if (! can_set_background_image (panel->toplevel, uri, NULL))
+		return drop_uri (panel, pos, uri, PANEL_ICON_UNKNOWN);
+
+	dialog = image_drop_dialog (panel->toplevel, uri);
+
+	g_object_set_data (G_OBJECT (dialog), "pos", GINT_TO_POINTER (pos));
+	g_object_set_data_full (G_OBJECT (dialog), "uri", g_strdup (uri), g_free);
+
+	g_signal_connect (dialog, "response",
+	                  G_CALLBACK (image_drop_dialog_response), panel);
+
+	gtk_widget_show (dialog);
+
+	return TRUE;
+}
+
 static gboolean
 drop_urilist (PanelWidget *panel,
 	      int          pos,
 	      char        *urilist)
 {
 	char     **uris;
+	guint      n_uris;
 	gboolean   success;
 	int        i;
 
 	uris = g_uri_list_extract_uris (urilist);
+	n_uris = g_strv_length (uris);
 
 	success = TRUE;
 	for (i = 0; uris[i]; i++) {
@@ -649,9 +799,9 @@ drop_urilist (PanelWidget *panel,
 			can_exec = g_file_info_get_attribute_boolean (info,
 								      G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE);
 
-			if (mime &&
-			    g_str_has_prefix (mime, "image")) {
-				if (!set_background_image_from_uri (panel->toplevel, uri))
+			if (n_uris == 1 &&
+			    mime && g_str_has_prefix (mime, "image")) {
+				if (! ask_about_image_drop (panel, pos, uri))
 					success = FALSE;
 			} else if (mime &&
 				   (!strcmp (mime, "application/x-mate-app-info") ||
@@ -684,7 +834,7 @@ drop_urilist (PanelWidget *panel,
 				success = FALSE;
 		}
 
-		g_object_unref (info);
+		g_clear_object (&info);
 		g_object_unref (file);
 	}
 
@@ -709,7 +859,7 @@ drop_internal_icon (PanelWidget *panel,
 
 	if (action == GDK_ACTION_MOVE)
 		old_launcher = find_launcher (icon_name);
-	
+
 	if (!panel_launcher_create_copy (panel->toplevel, pos, icon_name))
 		return FALSE;
 
@@ -1045,7 +1195,7 @@ drag_drop_cb (GtkWidget	        *widget,
 	return TRUE;
 }
 
-static void  
+static void
 drag_leave_cb (GtkWidget	*widget,
 	       GdkDragContext   *context,
 	       guint             time,
@@ -1155,9 +1305,9 @@ drag_data_recieved_cb (GtkWidget	*widget,
 	panel_widget = panel_toplevel_get_panel_widget (PANEL_TOPLEVEL (widget));
 
 	pos = panel_widget_get_cursorloc (panel_widget);
-	
-	/* 
-	 * -1 passed to mate_panel_applet_register will turn on 
+
+	/*
+	 * -1 passed to mate_panel_applet_register will turn on
 	 * the insert_at_pos flag for panel_widget_add_full,
 	 * which will not place it after the first applet.
 	 */
@@ -1212,7 +1362,7 @@ panel_setup (PanelToplevel *toplevel)
 	pd->deactivate_idle = 0;
 
 	panel_list = g_slist_append (panel_list, pd);
-	
+
 	g_object_set_data (G_OBJECT (toplevel), "PanelData", pd);
 
 	panel_lockdown_notify_add (G_CALLBACK (panel_recreate_context_menu),
@@ -1240,7 +1390,7 @@ panel_setup (PanelToplevel *toplevel)
 
 	g_signal_connect_swapped (toplevel, "notify::orientation",
 				  G_CALLBACK (panel_orient_change), panel_widget);
- 
+
 	g_signal_connect (toplevel, "destroy", G_CALLBACK (panel_destroy), pd);
 
 	return pd;
@@ -1292,7 +1442,7 @@ panel_delete_without_query (PanelToplevel *toplevel)
 		panel_profile_delete_object (info);
 	} else
 		panel_profile_delete_toplevel (toplevel);
-} 
+}
 
 static void
 panel_deletion_response (GtkWidget     *dialog,
@@ -1340,9 +1490,9 @@ panel_deletion_dialog (PanelToplevel *toplevel)
 			GTK_MESSAGE_WARNING,
 			GTK_BUTTONS_NONE,
 			"%s", text1);
-	
+
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-	                                          "%s", text2);	
+	                                          "%s", text2);
 
 	panel_dialog_add_button (GTK_DIALOG (dialog),
 				 _("_Cancel"), "process-stop",
