@@ -1,7 +1,7 @@
 #ifndef __XSTUFF_H__
 #define __XSTUFF_H__
 
-#ifdef PACKAGE_NAME // only check HAVE_X11 if config.h has been included
+#ifdef PACKAGE_NAME /* only check HAVE_X11 if config.h has been included */
 #ifndef HAVE_X11
 #error file should only be included when HAVE_X11 is enabled
 #endif
